/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import java.io.*;
import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CFonction extends CCalculBase {
	
  int operateurFonction;
  CCalculBase operande;
  boolean dejaPositionne; // ne servira que pour un appel fonction
  // Rand() de gnration d'un nombre alatoire.
  double valeurDejaPrise; // Comme son nom l'indique
  double valeurDejaPriseImaginaire; // Partie imaginaire pour les fonctions complexes
  // Attibut rajout pour optimisation
  Complexe z1loc = new Complexe();
  /** Creates new CFonction */
  public CFonction() {
  }
  
  // Constructeur pour clonage
  public CFonction(CListeObjets liste, int operateurFonctionInit, CCalculBase operandeInit) {
    super(liste);
    operateurFonction = operateurFonctionInit;
    operande = operandeInit;
  }
  
  @Override
  protected long nature() {
  	return natFonction;
  }  
  
  // Pour la version 3.0, les indices de fonctions complexes ont t dcales pour faire de la place
  // d'o changement de version
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  @Override
  public int nombreVariables() {
  	return 1;
  }
  
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande = operande.getClone(listeSource, listeCible); 
    CFonction ptcalc = new CFonction(listeCible, operateurFonction, cloneOperande);
    // Les trois lignes suivantes ajoutes version 4.4 pour que les valeurs rand ne changent pas dans l'annuler-refaire
    ptcalc.dejaPositionne = dejaPositionne;
    ptcalc.valeurDejaPrise = valeurDejaPrise;
    ptcalc.valeurDejaPriseImaginaire = valeurDejaPriseImaginaire;
    return ptcalc;
  }

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	operande.initialisePourDependance();
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || operande.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return operande.dependDePourReclassement(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    // Attention : modifi en java par rapport au C++
    if (operateurFonction == OperateurFonction.Rand) 
    {
      if (p instanceof CVariableBornee) return true;
        else return operande.dependDePourBoucle(p);
    }
    else return operande.dependDePourBoucle(p);
  }

  @Override
  public boolean existe(boolean infoRandom) {
    return operande.existe(infoRandom);
  }

  public double resultatBase(boolean infoRandom, double x) throws ErreurCalculException {
    switch (operateurFonction)
    {
      case OperateurFonction.Abso:
        return Math.abs(x);
      case OperateurFonction.Enti:
        return Math.floor(x);
      case OperateurFonction.Raci:
      case OperateurFonction.Raci2:
        if (x >= 0)
          return Math.sqrt(x);
        else throw new ErreurCalculException();
      case OperateurFonction.Sinu:
        if (listeProprietaire.uniteAngle == UniteAngle.Radian) return CCalculBase.sinus(x);
        else return CCalculBase.sinus(x*Kernel.ConvDegRad);
      case OperateurFonction.Cosi:
        if (listeProprietaire.uniteAngle == UniteAngle.Radian) return CCalculBase.cosinus(x);
        else return CCalculBase.cosinus(x*Kernel.ConvDegRad);
      case OperateurFonction.Tang:
        if (listeProprietaire.uniteAngle == UniteAngle.Radian) return CCalculBase.tangente(x);
        else {
          return CCalculBase.tangente(x*Kernel.ConvDegRad);
        }
      case OperateurFonction.Logn:
        if (x>0) return Math.log(x);
        else throw new ErreurCalculException();
      case OperateurFonction.Expo:
        return Math.exp(x);
      case OperateurFonction.ATan:
        if (listeProprietaire.uniteAngle == UniteAngle.Radian) return Math.atan(x);
        else return Math.atan(x)*Kernel.ConvRadDeg;
      case OperateurFonction.ACos:
        if (listeProprietaire.uniteAngle == UniteAngle.Radian) return Math.acos(x);
        else return Math.acos(x)*Kernel.ConvRadDeg;
      case OperateurFonction.ASin:
        if (listeProprietaire.uniteAngle == UniteAngle.Radian) return Math.asin(x);
        else return Math.asin(x)*Kernel.ConvRadDeg;
      case OperateurFonction.Cosh:
        return (Math.exp(x) + Math.exp(-x))/2;
      case OperateurFonction.Sinh:
        return (Math.exp(x) - Math.exp(-x))/2;
      case OperateurFonction.Tanh:
        return (Math.exp(2*x) - 1)/(Math.exp(2*x) + 1);
      case OperateurFonction.ArgCh:
        if (x >= 1) return Math.log(x + Math.sqrt(x*x-1));
        else throw new ErreurCalculException();
      case OperateurFonction.ArgSh:
    		return Math.log(x + Math.sqrt(x*x+1));
      case OperateurFonction.ArgTh:
        if ((x > -1) && (x < 1 )) return 0.5*Math.log((1+x)/(1-x));
        else throw new ErreurCalculException();
      case OperateurFonction.Rand:
        if (infoRandom)
        {
          valeurDejaPrise = Math.random();
          dejaPositionne = true;
          return valeurDejaPrise;
        }
        else
        {
          if (!dejaPositionne)
          {
            valeurDejaPrise = Math.random();
            dejaPositionne = true;
            return valeurDejaPrise;
          }
          else return valeurDejaPrise;
        }
      case OperateurFonction.fact:
      	if ((x < 0) || (x != Math.round(x))) throw new ErreurCalculException();
      	if (x <= 32) {
      		double p = 1;
      		for (int i = 2; i <= x; i++) {
      			p = p*i;
      		}
      		return (double) p;
      	}
      	else {
      		double x1 = x;
      		double d = 1 + 1/(12*x1);
      		x1 *= x;
      		d += 1/(288*x1);
      		x1 *= x;
      		d -= 139/(51840*x1);
      		x1 *= x;
      		d -= 571/(2488320*x1);
      		x1*=x;
      		d -= 163879/(209018880*x1);
      		// double d = 1+1/(12*x)+1/(288*x*x)-139/(51840*x*x*x)-571/(2488320*x*x*x*x)+163879/(209018880*x*x*x*x*x);
      		return Math.exp(0.5*Math.log(2*Math.PI*x)+x*(Math.log(x)-1))*d;
      	}      	
      default : throw new ErreurCalculException();
    }// switch OperateurFonction}
  }
  public double resultat(boolean infoRandom) throws ErreurCalculException {
  	switch (operateurFonction) {
  	case OperateurFonction.Left :
  	case OperateurFonction.LeftC : // Rajout version 4.7.2
    	CCalculBase mg = operande.membreGauche();
  		if (mg == null) {
  			// new ErreurCalculException(); // Erruer corrige version 4.7.5.2
  			throw new ErreurCalculException();
  		}
  		else return operande.membreGauche().resultat(infoRandom);
  	case OperateurFonction.Right :
  	case OperateurFonction.RightC : // Rajout version 4.7.2
    	CCalculBase md = operande.membreDroit();
  		if (md == null) {
  			throw new ErreurCalculException();
  		}
  		return operande.membreDroit().resultat(infoRandom);
  	default :
	    double x1 = operande.resultat(infoRandom);
	    return resultatBase(infoRandom, x1);
  	}
  }

  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
  	switch (operateurFonction) {
  	case OperateurFonction.Left :
  	case OperateurFonction.LeftC : // Rajout version 4.7.2
    	CCalculBase mg = operande.membreGauche();
  		if (mg == null) {
  			throw new ErreurCalculException();
  		}
  		return operande.membreGauche().resultatFonction(infoRandom, valeurParametre);
  	case OperateurFonction.Right :
  	case OperateurFonction.RightC : // Rajout version 4.7.2
    	CCalculBase md = operande.membreDroit();
  		if (md == null) {
  			throw new ErreurCalculException();
  		}
  		return operande.membreDroit().resultatFonction(infoRandom, valeurParametre);
  	default :
	    double x1 = operande.resultatFonction(infoRandom, valeurParametre);
	    return resultatBase(infoRandom, x1);
  	}
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
  	switch (operateurFonction) {
  	case OperateurFonction.Left :
  	case OperateurFonction.LeftC : // Rajout version 4.7.2
    	CCalculBase mg = operande.membreGauche();
  		if (mg == null) {
  			throw new ErreurCalculException();
  		}
  		return operande.membreGauche().resultatFonction(infoRandom, valeurParametre);
  	case OperateurFonction.Right :
  	case OperateurFonction.RightC : // Rajout version 4.7.2
    	CCalculBase md = operande.membreDroit();
  		if (md == null) {
  			throw new ErreurCalculException();
  		}
  		return operande.membreDroit().resultatFonction(infoRandom, valeurParametre);
  	default :
	    double x1 = operande.resultatFonction(infoRandom, valeurParametre);
	    return resultatBase(infoRandom, x1);
  	}
  }
  
  void resultatComplexeBase(boolean infoRandom, Complexe z, Complexe zRes)  throws ErreurCalculException {
    switch (operateurFonction)
    {
      case OperateurFonction.Abs:
              zRes.y = 0;
              zRes.x = z.module();
              return;
      case OperateurFonction.Sin:
              z.sinComp(listeProprietaire.uniteAngle, zRes);
              return;
      case OperateurFonction.Cos:
              z.cosComp(listeProprietaire.uniteAngle, zRes);
              return;
      case OperateurFonction.Tan:
              z.tanComp(listeProprietaire.uniteAngle, zRes);
              return;
      case OperateurFonction.Log:
              z.lnComp(listeProprietaire.uniteAngle, zRes);
              return;
      case OperateurFonction.Exp:
              z.expComp(listeProprietaire.uniteAngle, zRes);
              return;
      case OperateurFonction.Ch:
              z.chComp(listeProprietaire.uniteAngle, zRes);
              return;
      case OperateurFonction.Sh:
              z.shComp(listeProprietaire.uniteAngle,zRes);
              return;
      case OperateurFonction.Th:
              z.thComp(listeProprietaire.uniteAngle,zRes);
              return;
      case OperateurFonction.Rnd:
      	zRes.y = 0;
        if (infoRandom)
        {
          valeurDejaPrise = Math.random();
          dejaPositionne = true;
          zRes.x = valeurDejaPrise;
        }
        else
        {
          if (!dejaPositionne)
          {
            valeurDejaPrise = Math.random();
            dejaPositionne = true;
            zRes.x = valeurDejaPrise;
          }
          else zRes.x = valeurDejaPrise;
        }
        break;
      case OperateurFonction.Conj:
        zRes.x = z.x;
        zRes.y = -z.y;
        return;
      case OperateurFonction.Arg:
        if ((z.x == 0) && (z.y == 0))
        {
          zRes.x = 0;
          zRes.y = 0;
          throw new ErreurCalculException();
        }
        else
        {
          zRes.x = z.argument(listeProprietaire.uniteAngle);
          zRes.y = 0;
          return;
        }
      case OperateurFonction.Re:
        zRes.x = z.x;
        zRes.y = 0;
        return;
      case OperateurFonction.Im:
        zRes.x = z.y;
        zRes.y = 0;
        return;
      case OperateurFonction.Rac:
      case OperateurFonction.Rac2:
        zRes.x = 0;
        zRes.y = 0;
        if ((z.y != 0) || (z.x <0))
        {
          throw new ErreurCalculException();
        }
        else
        {
          zRes.x = Math.sqrt(z.x);
          return;
        }
      case OperateurFonction.EntiC :
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          zRes.x = Math.floor(z.x);
          return;
        }
      case OperateurFonction.ACosC :
        // le rsultat n'existe que si l'argument est rel
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          zRes.x = Math.acos(z.x);
          return;
        }
      case OperateurFonction.ASinC :
        // le rsultat n'existe que si l'argument est rel
        // le rsultat n'existe que si l'argument est rel
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          zRes.x = Math.asin(z.x);
          return;
        }
      case OperateurFonction.ATanC :
        // le rsultat n'existe que si l'argument est rel
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          zRes.x = Math.atan(z.x);
          return;
        }
      case OperateurFonction.ArgChC :
        // le rsultat n'existe que si l'argument est rel
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          if (z.x >= 1) zRes.x = Math.log(z.x + Math.sqrt(z.x*z.x-1));
          else throw new ErreurCalculException();
        }
        break;
      case OperateurFonction.ArgShC :
        // le rsultat n'existe que si l'argument est rel
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          zRes.x = Math.log(z.x + Math.sqrt(z.x*z.x+1));
          return;
        }
      case OperateurFonction.ArgThC :
        // le rsultat n'existe que si l'argument est rel
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          if ((z.x > -1) && (z.x < 1 )) zRes.x = 0.5*Math.log((1+z.x)/(1-z.x));
          else throw new ErreurCalculException();
        }
        
      case OperateurFonction.factC:
        zRes.x = 0;
        zRes.y = 0;
        if (z.y != 0)
        {
          throw new ErreurCalculException();
        }
        else {
		     	if ((z.x < 0) ||(z.x != Math.round(z.x))) throw new ErreurCalculException();
		      	if (z.x <= 32) {
		      		double p = 1;
		      		for (int i = 2; i <= z.x; i++) {
		      			p = p*i;
		      		}
		      		zRes.x = p;
		      	}
		      	else {
		      		double x = z.x;
		      		double x1 = x;
		      		double d = 1 + 1/(12*x1);
		      		x1 *= x;
		      		d += 1/(288*x1);
		      		x1 *= x;
		      		d -= 139/(51840*x1);
		      		x1 *= x;
		      		d -= 571/(2488320*x1);
		      		x1*=x;
		      		d -= 163879/(209018880*x1);
		      		// double d = 1+1/(12*x)+1/(288*x*x)-139/(51840*x*x*x)-571/(2488320*x*x*x*x)+163879/(209018880*x*x*x*x*x);
		      		zRes.x = Math.exp(0.5*Math.log(2*Math.PI*x)+x*(Math.log(x)-1))*d;		      		
		      	}

        }
    }; // switch OperateurFonction}    
  }
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
  	switch (operateurFonction) {
	  	case OperateurFonction.Left :
	  	case OperateurFonction.LeftC : // Rajout version 4.7.2
	  		operande.membreGauche().resultatComplexe(infoRandom, zRes);
	  		break; // Rajout version 4.7.2
	  	case OperateurFonction.Right :
	  	case OperateurFonction.RightC : // Rajout version 4.7.2
	  		operande.membreDroit().resultatComplexe(infoRandom, zRes);
	  		break; // Rajout version 4.7.2
	  	default :
		    operande.resultatComplexe(infoRandom, z1loc);
		    resultatComplexeBase(infoRandom, z1loc, zRes);
  	}
  }
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
    throws ErreurCalculException {
  	switch (operateurFonction) {
	  	case OperateurFonction.Left :
	  	case OperateurFonction.LeftC : // Rajout version 4.7.2
	    	CCalculBase mg = operande.membreGauche();
	  		if (mg == null) {
	  			throw new ErreurCalculException();
	  		}
	  		mg.resultatFonctionComplexe(infoRandom, valeurParametre, zRes);
	  		break; // Rajout version 4.7.2
	  	case OperateurFonction.Right :
	  	case OperateurFonction.RightC : // Rajout version 4.7.2
	    	CCalculBase md = operande.membreDroit();
	  		if (md == null) {
	  			throw new ErreurCalculException();
	  		}
	  		md.resultatFonctionComplexe(infoRandom, valeurParametre, zRes);
	  		break; // Rajout version 4.7.2
	  	default :
		    operande.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
		    resultatComplexeBase(infoRandom, z1loc, zRes);
  	}
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
    throws ErreurCalculException {
  	switch (operateurFonction) {
  	case OperateurFonction.Left :
  	case OperateurFonction.LeftC : // Rajout version 4.7.2
    	CCalculBase mg = operande.membreGauche();
  		if (mg == null) {
  			throw new ErreurCalculException();
  		}
  		mg.resultatFonctionComplexe(infoRandom, valeurParametre, zRes);
  		break; // Rajout version 4.7.2
  	case OperateurFonction.Right :
  	case OperateurFonction.RightC : // Rajout version 4.7.2
    	CCalculBase md = operande.membreDroit();
  		if (md == null) {
  			throw new ErreurCalculException();
  		}
  		md.resultatFonctionComplexe(infoRandom, valeurParametre, zRes);
  		break; // Rajout version 4.7.2
  	default :
	    operande.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
	    resultatComplexeBase(infoRandom, z1loc, zRes);
  	}
  }
  
  /*
  public boolean  dependDeVariable(int ind) {
    return operande.dependDeVariable(ind);
  }
  */
  //Amlioration version,4.7.8.1 pour les fonctions droite et gauche
  public boolean  dependDeVariable(int ind) {
  	switch (operateurFonction) {
	  	case OperateurFonction.Left :
	  	case OperateurFonction.LeftC : // Rajout version 4.7.2
	    	return operande.membreGauche().dependDeVariable(ind);
	  	case OperateurFonction.Right :
	  	case OperateurFonction.RightC : // Rajout version 4.7.2
	    	return operande.membreDroit().dependDeVariable(ind);
	    default :
	    	return operande.dependDeVariable(ind);
  	}
  }
  public CCalculBase getCopie() {
   	return new CFonction(listeProprietaire, operateurFonction, operande.getCopie());
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String cha = operande.chaineCalculSansPar(varFor);

  	switch (operateurFonction)
  	{
  		case OperateurFonction.Abso:
  		case OperateurFonction.Abs:
  			return getString("abs") + parouv + cha + parfer;
  		case OperateurFonction.Enti:
  		case OperateurFonction.EntiC:
  			return getString("intgr") + parouv + cha + parfer;
  		case OperateurFonction.Raci:
  		case OperateurFonction.Rac:
  			return getString("sqrt") + parouv + cha + parfer;
  		case OperateurFonction.Raci2:
  		case OperateurFonction.Rac2:
  			return getString("rac") + parouv + cha + parfer;
  		case OperateurFonction.Sinu:
  		case OperateurFonction.Sin:
  			return getString("sin") + parouv + cha + parfer;
  		case OperateurFonction.Cosi:
  		case OperateurFonction.Cos:
  			return getString("cos") + parouv + cha + parfer;
  		case OperateurFonction.Tang:
  		case OperateurFonction.Tan:
  			return getString("tan") + parouv + cha + parfer;
  		case OperateurFonction.Logn:
  		case OperateurFonction.Log:
  			return getString("ln") + parouv + cha + parfer;
  		case OperateurFonction.Expo:
  		case OperateurFonction.Exp:
  			return getString("exp") + parouv + cha + parfer;
  		case OperateurFonction.ATan:
  		case OperateurFonction.ATanC:
  			return getString("arctan") + parouv + cha + parfer;
  		case OperateurFonction.ACos:
  		case OperateurFonction.ACosC:
  			return getString("arccos") + parouv + cha + parfer;
  		case OperateurFonction.ASin:
  		case OperateurFonction.ASinC:
  			return getString("arcsin") + parouv + cha + parfer;
  		case OperateurFonction.Cosh:
  		case OperateurFonction.Ch:
  			return getString("ch") + parouv + cha + parfer;
  		case OperateurFonction.Sinh:
  		case OperateurFonction.Sh:
  			return getString("sh") + parouv + cha + parfer;
  		case OperateurFonction.Tanh:
  		case OperateurFonction.Th:
  			return getString("th") + parouv + cha + parfer;
  		case OperateurFonction.fact:
  		case OperateurFonction.factC:
  			return getString("fact") + parouv + cha + parfer;
  		// Deux ajouts version 4.6
  		case OperateurFonction.Left:
  		case OperateurFonction.LeftC:
  				return getString("left") + parouv + cha + parfer;
  		case OperateurFonction.Right:
  		case OperateurFonction.RightC:
  				return getString("right") + parouv + cha + parfer;
  		// A revoir ici. Pour la version 1.5 j'ai oublie le ArgCh
  		case OperateurFonction.ArgCh:
  		case OperateurFonction.ArgChC:
  			return getString("argch") + parouv + cha + parfer;
  		case OperateurFonction.ArgSh:
  		case OperateurFonction.ArgShC:
  			return getString("argsh") + parouv + cha + parfer;
  		case OperateurFonction.ArgTh:
  		case OperateurFonction.ArgThC:
  			return getString("argth") + parouv + cha + parfer;
  		case OperateurFonction.Rand:
  		case OperateurFonction.Rnd:
  			return getString("rand") + parouv + cha + parfer;
  		case OperateurFonction.Conj:
  			return getString("conj") + parouv + cha + parfer;
  		case OperateurFonction.Arg:
  			return getString("arg") + parouv + cha + parfer;
  		case OperateurFonction.Re:
  			return getString("re") + parouv + cha + parfer;
  		case OperateurFonction.Im:
  			return getString("im") + parouv + cha + parfer;

  		default:
  			return "";
  	} // switch OperateurFonction}}
  }
  
  @Override
	public String chaineLatex(String[] varFor) {
  	String cha = operande.chaineLatexSansPar(varFor);

  	switch (operateurFonction)
  	{
  		case OperateurFonction.Abso:
  		case OperateurFonction.Abs:
  			return "\\left| {"+ cha + "} \\right|";
  		case OperateurFonction.Enti:
  		case OperateurFonction.EntiC:
  			return getString("intgr") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Raci:
  		case OperateurFonction.Rac:
  			return "\\sqrt{" + cha + "}";
  		case OperateurFonction.Raci2:
  		case OperateurFonction.Rac2:
  			return "\\sqrt{" + cha + "}";
  		case OperateurFonction.Sinu:
  		case OperateurFonction.Sin: // Modifi version 4.7.7 pour internationalisation
  			// return "\\sin" + parouvLatex + cha + parferLatex;
  			return getString("sin") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Cosi:
  		case OperateurFonction.Cos: // Modifi version 4.7.7 pour internationalisation
  			return getString("cos") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Tang:
  		case OperateurFonction.Tan: // Modifi version 4.7.7 pour internationalisation
  			return getString("tan") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Logn:
  		case OperateurFonction.Log: // Modifi version 4.7.7 pour internationalisation
  			return getString("ln") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Expo:
  		case OperateurFonction.Exp:
  			return "e^" + "{" + cha + "}";
  		case OperateurFonction.ATan:
  		case OperateurFonction.ATanC: // Modifi version 4.7.7 pour internationalisation
  			return getString("arctan") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.ACos:
  		case OperateurFonction.ACosC: // Modifi version 4.7.7 pour internationalisation
  			return getString("arccos") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.ASin:
  		case OperateurFonction.ASinC: // Modifi version 4.7.7 pour internationalisation
  			return getString("arcsin") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Cosh:
  		case OperateurFonction.Ch: // Modifi version 4.7.7 pour internationalisation
  			return getString("cosh") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Sinh:
  		case OperateurFonction.Sh: // Modifi version 4.7.7 pour internationalisation
  			return getString("sinh") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Tanh:
  		case OperateurFonction.Th: // Modifi version 4.7.7 pour internationalisation
  			return getString("tanh") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.fact:
  		case OperateurFonction.factC:
  			return cha + "!"; // A revoir
  		// Deux ajouts version 4.6
  		case OperateurFonction.Left:
  		case OperateurFonction.LeftC:
  				return getString("Left") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Right:
  		case OperateurFonction.RightC:
  				return getString("Left") + parouvLatex + cha + parferLatex;
  		// A revoir ici. Pour la version 1.5 j'ai oublie le ArgCh
  		case OperateurFonction.ArgCh:
  		case OperateurFonction.ArgChC:
  			return getString("argch") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.ArgSh:
  		case OperateurFonction.ArgShC:
  			return getString("argsh") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.ArgTh:
  		case OperateurFonction.ArgThC:
  			return getString("argth") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Rand:
  		case OperateurFonction.Rnd:
  			return getString("rand") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Conj:
  			return "\\overline {" + cha + "}";
  		case OperateurFonction.Arg:
  			return getString("arg") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Re:
  			return getString("re") + parouvLatex + cha + parferLatex;
  		case OperateurFonction.Im:
  			return getString("im") + parouvLatex + cha + parferLatex;

  		default:
  			return "";
  	} // switch OperateurFonction}}
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return operande.estDefiniParObjetDans(listeOb);
	}	   

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    operateurFonction = inps.readByte();
    if ((nVersion == 1) && (operateurFonction >= 19)) operateurFonction += 45;
    operande = (CCalculBase) inps.readCObject(list);
		if ((listeProprietaire.numeroVersion > 3) && (operateurFonction ==OperateurFonction.Rand))
		{
			valeurDejaPrise= inps.readCDouble();
			dejaPositionne = true;
    }
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operateurFonction = inps.readByte();
    // A partir de la version 3.0, dcalage des indices de fonctions complexes
    if ((nVersion == 1) && (operateurFonction >= 19)) operateurFonction += 45;
    operande = (CCalculBase) inps.readObject(list);
		if (operateurFonction == OperateurFonction.Rand)
		{
			valeurDejaPrise= inps.readDouble();
			dejaPositionne = true;
    }
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(operateurFonction);
    oups.writeObject(operande);
		if (operateurFonction == OperateurFonction.Rand)
		{
      oups.writeDouble(valeurDejaPrise);
    }
  }
  
  @Override
	public boolean estConstant() {
		return (operateurFonction != OperateurFonction.Rand) && operande.estConstant();
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return operande.deriveePossible(indiceVariable);
  }
  
  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CFonction(listeProprietaire, operateurFonction, operande.calculAvecValeursRemplacees());
  }
  
}
