/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import java.io.*;
import java.awt.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 *
 * @author yves.bitton
 */
public class CGrapheSuiteRecComplexe extends CElementLigne {
	public int motif;
	public boolean pointsRelies;
	public  CSuiteRecComplexe suiteAssociee;
	public  CRepere repereAssocie;
	private CPointDansRepere point1;
	private CPointDansRepere point2; // Sert  tracer les lignes
	private CSegment seg;
  private boolean infr;
  private DimensionFenetre dimf;
  
  /** Creates a new instance of CGrapheSuiteRecComplexe */
  public CGrapheSuiteRecComplexe() {
  }
  
  // Constructeur pour clonage
  public CGrapheSuiteRecComplexe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, CSuiteRecComplexe suiteAssocieeInit, CRepere repereAssocieInit,
      int motifInit,  boolean pointsReliesInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    suiteAssociee = suiteAssocieeInit;
    repereAssocie = repereAssocieInit;
    motif = motifInit;
    pointsRelies = pointsReliesInit;
    point1 = new CPointDansRepere(listeProprietaire, couleur, motif, repereAssocie);
    point2 = new CPointDansRepere(listeProprietaire, couleur, motif, repereAssocie);
    seg = new CSegment(liste, point1, point2);
  }
 
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(suiteAssociee);
    int ind2 = listeSource.indexOf(repereAssocie);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CGrapheSuiteRecComplexe(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CSuiteRecComplexe)(listeCible.get(ind1, CSuiteRecComplexe.class)),
        (CRepere)(listeCible.get(ind2, CRepere.class)), motif, pointsRelies);
    return ptelb;
  }
  
  /**
   * Lorsqu'on cre un graphe de suite rcurrente, le repre n'a pas encore t choisi.
   * Il faut aprs la cration appeler cette fonction pour que les points reprs utiliss de faon interne
   * soient initialiss
   * @param rep
   */
  public void associeARepere(CRepere rep) {
  	repereAssocie = rep;
  	point1.rep = rep;
  	point2.rep = rep;
  }  
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || suiteAssociee.dependDe(p) || repereAssocie.dependDe(p));
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this)) || suiteAssociee.dependDePourBoucle(p) || repereAssocie.dependDePourBoucle(p);
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = ((suiteAssociee.existe) && (repereAssocie.existe));
  // Cette classe doit mmoriser dans positionne() les valeurs de infoRandom et de dimfen
    infr = infoRandom;
    dimf = dimfen;
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {

    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    
    pDC.setStroke(Pinceau.CreePinceau(styleTrait));
    pDC.setColor(couleur);
    seg.donneCouleur(couleur);
    seg.donneStyle(style);
    point1.donneMotif(motif);
    point1.donneCouleur(couleur);
    
    for (int i = 0; i <= suiteAssociee.indiceDernierTermeExistant; i++)
    {
      point1.abs.donneValeur(suiteAssociee.valeurs[i].x);
      point1.ord.donneValeur(suiteAssociee.valeurs[i].y);
      point1.positionne(infr, dimf);
      point1.trace(pDC, couleurFond, transparence);
      if (pointsRelies && i != suiteAssociee.indiceDernierTermeExistant)
      {
        point2.abs.donneValeur(suiteAssociee.valeurs[i+1].x);
        point2.ord.donneValeur(suiteAssociee.valeurs[i+1].y);
        point2.positionne(infr, dimf);
        seg.positionne(infr, dimf);
        if (seg.existe) seg.trace(pDC, couleurFond, transparence);
      }
    }
  }
  
  public double distancePoint(double xp, double yp, boolean masquage) {
  	double dist;

  	if (!existe || (masquage && masque)) return -1;
  	if (pointsRelies)
  	{
  		for (int i = 0; i <= suiteAssociee.indiceDernierTermeExistant-1; i++)
  		{
  			point1.abs.donneValeur(suiteAssociee.valeurs[i].x);
  			point1.ord.donneValeur(suiteAssociee.valeurs[i].y);
  			point2.abs.donneValeur(suiteAssociee.valeurs[i+1].x);
  			point2.ord.donneValeur(suiteAssociee.valeurs[i+1].y);
  			point1.positionne(false, dimf);
  			point2.positionne(false, dimf);
  			seg.positionne(false, dimf);
  			dist = seg.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		} // for
  	}
  	else
  	{
  		for (int i = 0; i <= suiteAssociee.indiceDernierTermeExistant; i++)
  		{
  			point1.abs.donneValeur(suiteAssociee.valeurs[i].x);
  			point1.ord.donneValeur(suiteAssociee.valeurs[i].y);
  			point1.positionne(false, dimf);
  			dist = point1.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		}
  	}
  	return -1;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NGrapheSuiteRecComplexe;
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CGrapheSuiteRecComplexe pt;
    if (p.getClass() == this.getClass()) {
      pt = (CGrapheSuiteRecComplexe) p;
   		return (repereAssocie == pt.repereAssocie) && (suiteAssociee == pt.suiteAssociee);
    }
    else return false;
  }        
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    suiteAssociee = (CSuiteRecComplexe) (list.get(ind1,CSuiteRecComplexe.class));
    repereAssocie = (CRepere) (list.get(ind2,CRepere.class));
    pointsRelies = inps.readBoolean();
    motif = inps.readByte();
    point1 = new CPointDansRepere(listeProprietaire, couleur, motif, repereAssocie);
    point2 = new CPointDansRepere(listeProprietaire, couleur, motif, repereAssocie);
    seg = new CSegment(list, point1, point2);
   }
  /* Enregistrement dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    suiteAssociee = (CSuiteRecComplexe) (list.get(ind1,CSuiteRecComplexe.class));
    repereAssocie = (CRepere) (list.get(ind2,CRepere.class));
    pointsRelies = inps.readBoolean();
    motif = inps.readByte();
    point1 = new CPointDansRepere(listeProprietaire, couleur, motif, repereAssocie);
    point2 = new CPointDansRepere(listeProprietaire, couleur, motif, repereAssocie);
    seg = new CSegment(list, point1, point2);
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(suiteAssociee);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(repereAssocie);
    oups.writeInt(ind2);
    oups.writeBoolean(pointsRelies);
    oups.writeByte(motif);
  }
}
