/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CLieuDePoints extends CLieuDeBase {
  CPointLie pointLieGenerateur;// le point li gnrateur du lieu
  
  /** Creates new CLieuDePoints */
  public CLieuDePoints() {
  }
  
  // Constructeur pour clonage
  public CLieuDePoints(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, CPointAncetre pointATracerInit, InfoLieu infoLieuInit,
    CPointLie pointLieGenerateurInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, pointATracerInit, infoLieuInit);
    pointLieGenerateur = pointLieGenerateurInit;
    // Rajout version 3.4.2
    // etablitListeElementsAncetres(); Inutile et provoquait un bug sous version 4.4 en appelant getClone
  }
  
  /**
   * A partir de la version 3.4.2, changement de version (version 1 -> 2) car on enregistre dans le flux la CSousListeObjetstres
   * contenant les pointeurs sur les lments anctres
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    InfoLieu infoLieuClone = new InfoLieu(infoLieu);
    
    int ind1 = listeSource.indexOf(pointATracer);
    int ind2 = listeSource.indexOf(pointLieGenerateur);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CLieuDePoints(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      infoLieuClone, (CPointLie)(listeCible.get(ind2, CPointLie.class)));
    if (listeCible.getClass() != CPrototype.class) {
	   ((CLieuDePoints)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire,
	      ptelb.listeProprietaire);
    }
    return ptelb;
  }
  
  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   */
  @Override
  public void metAJour() {
  	super.metAJour();
  	// Bug corrig version 4.0 : il faut mettre  jour la liste d'lments anctres car metAJour est appel quand
  	// on modifie un obejt dont dpend cet objet
  	// if (nVersion == 1) etablitListeElementsAncetres();
  	etablitListeElementsAncetres();
  }
    
  // Modification pour la version 2.0 : quand des points sont isols ou n'existent pas on met leur abscissesCoordonneesPoints
  // et leurs ordonneesCoordonneesPoints  intMax de faon  ce que la distance entre un point et cette position ne puisse
  // pas tre suffisamment petite pour tre considre comme proche.
  // Cette modification est ncessaire  cause de l'introduction du point li  un lieu de points par point li
  // On doit pouvoir mesurer la distance entre le pointeur souris et les positions du point gnrateur du lieu
  // et calculer l'abscisse correspondante
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double abscisseMini,abscisseMaxi,abscisse,abscisseInitiale,pas,abscisseFinale;
    
    if (!pointLieGenerateur.existe) {
      existe = false;
      return;
    }
    abscisseMini = pointLieGenerateur.abscisseMinimale();
    abscisseMaxi = pointLieGenerateur.abscisseMaximale();
    if (abscisseMini == abscisseMaxi) {
      existe = false;
      return;
    } else {
      // On ordonne l'abscisse minimale et maximale
      if (abscisseMini > abscisseMaxi) {
        double w = abscisseMini;
        abscisseMini = abscisseMaxi;
        abscisseMaxi = w;
      }
    }
    abscisseInitiale = pointLieGenerateur.abscisse;
    // Modification pour la version 1.9.5
    int nbValeurs;
    if (pointLieGenerateur.lieALigneFermee()) {
    	nbValeurs = infoLieu.nombreDePoints;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi - pas;
    }
    else {
    	nbValeurs = infoLieu.nombreDePoints - 1;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi;
    }
    
    
    pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    abscisse = abscisseMini;
    
    boolean ligneEnCours = false;
    int indiceLigneEnCours = -1;
    
    // Modifi version 3.9.1 pour viter des erreurs d'arrondi
    int nb = infoLieu.nombreDePoints - 1;
    for (int i = 0; i < infoLieu.nombreDePoints; i++) {
      // Modifi version 3.9.1 pour viter des erreurs d'arrondi
    	// Bug corrig version 4.0
    	abscisse = (i == 0) ? abscisseMini : ((i < nb) ? abscisse + pas : abscisseFinale);
      pointLieGenerateur.donneAbscisse(abscisse);
      listeElementsAncetres.positionne(infoRandom, dimfen);
      if (!pointATracer.existe) {
      	abscissesCoordonneesPoints[i] = Integer.MAX_VALUE;
      	ordonneesCoordonneesPoints[i] = Integer.MAX_VALUE;
        if (ligneEnCours) {
          infoLignes[indiceLigneEnCours].nombrePoints =
            i - infoLignes[indiceLigneEnCours].indicePremierPoint;
          ligneEnCours = false;
        }
      } else {
        if (!(Kernel.testToile(pointATracer.x,pointATracer.y))) {
          if (ligneEnCours) {
            infoLignes[indiceLigneEnCours].nombrePoints =
              i - infoLignes[indiceLigneEnCours].indicePremierPoint;
            ligneEnCours = false;
          }
        }
        else {
          if (ligneEnCours) {
            if (infoLieu.gestionDiscontinuite) {
              if ((Math.abs(pointATracer.x - abscissesCoordonneesPoints[i-1]) > dimfen.longueurAbscisses)
              || (Math.abs(pointATracer.y - ordonneesCoordonneesPoints[i-1]) > dimfen.longueurOrdonnees)) {
                infoLignes[indiceLigneEnCours].nombrePoints =
                  i - infoLignes[indiceLigneEnCours].indicePremierPoint;
                ligneEnCours = false;
              }
              else {
                abscissesCoordonneesPoints[i] = (int) Math.round(pointATracer.x);
                ordonneesCoordonneesPoints[i] = (int) Math.round(pointATracer.y);
                infoLignes[indiceLigneEnCours].nombrePoints++;
              }
            }
            else {
              abscissesCoordonneesPoints[i] = (int) Math.round(pointATracer.x);
              ordonneesCoordonneesPoints[i] = (int) Math.round(pointATracer.y);
              infoLignes[indiceLigneEnCours].nombrePoints++;
            }
          }
          else {
            abscissesCoordonneesPoints[i] = (int) Math.round(pointATracer.x);
            ordonneesCoordonneesPoints[i] = (int) Math.round(pointATracer.y);
            indiceLigneEnCours++;
            ligneEnCours = true;
            infoLignes[indiceLigneEnCours].indicePremierPoint = i;
            infoLignes[indiceLigneEnCours].nombrePoints = 1;
          }
        }
      }
    } // for
    
    // On  remet le point anctre  sa position initiale
    pointLieGenerateur.donneAbscisse(abscisseInitiale);
    listeElementsAncetres.positionne(infoRandom, dimfen);
    nombreLignes = indiceLigneEnCours + 1;
    // Si le lieu n'est form d'aucune ligne ou s'il est form
    // de lignes avec un seul point, il n'existe pas
    if (nombreLignes == 0) {
      existe = false;
      return;
    }
    boolean auMoinsUneLigne = false;
    indicePremiereLigne = -1;
    nombreLignesDePlusieursPoints = 0;
    for (int j = 0; j <= indiceLigneEnCours; j++) {
      boolean ligneTrouvee = (infoLignes[j].nombrePoints >= 2);
      if (ligneTrouvee) {
        nombreLignesDePlusieursPoints++;
        if (!auMoinsUneLigne) indicePremiereLigne = j;
        auMoinsUneLigne = true;
      }
    }
    if (!auMoinsUneLigne) {
      existe = false;
      return;
    }
    // Si le lieu est dclar comme ferm, il n'existera que s'il
    // n'est form que d'une seule ligne avec au moins deux points
    
    if (infoLieu.fermeOuNon) {
    	// Modifi version 3.9. On accepte de fermer les lieux forms de plusieurs lignes
    	/*
      if (nombreLignes != 1) {
        existe = false;
        return;
      }
      */
      // On rajoute un dernier point gal de mme coordonnes
      // que le premier et on augmente le nombre de points de la ligne
    	// Modifi version 3.9. On ferme mme s'il y a plusieurs lignes
      // int indicePointRajoute = infoLignes[indicePremiereLigne].indicePremierPoint+
      //  infoLignes[indicePremiereLigne].nombrePoints;
      int indicePointRajoute = infoLignes[indiceLigneEnCours].indicePremierPoint+
      	infoLignes[indiceLigneEnCours].nombrePoints;
    	
      abscissesCoordonneesPoints[indicePointRajoute] =
        abscissesCoordonneesPoints[infoLignes[indicePremiereLigne].indicePremierPoint];
      ordonneesCoordonneesPoints[indicePointRajoute] =
        ordonneesCoordonneesPoints[infoLignes[indicePremiereLigne].indicePremierPoint];
      // infoLignes[indicePremiereLigne].nombrePoints++;
      infoLignes[indiceLigneEnCours].nombrePoints++;
    }
    existe = true;
  }
  
  public boolean confonduAvec(CElementBase p) {
    CLieuDePoints pt;
    if (p.getClass() == this.getClass()) {
      pt = (CLieuDePoints) p;
      return (pointATracer == pt.pointATracer) && (pointLieGenerateur == pt.pointLieGenerateur);
    } else return false;
  }
  
  public boolean dependDePourCapture(CElementBase p) {
    // Un lieu de points n'a pas  tre redessin si on bouge le point
    // li qui l'a engendr}
    return ((p == this) || (pointATracer.dependDe(p) && (p != pointLieGenerateur)));
  }
  
  public void etablitListeElementsAncetres() {
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (listeProprietaire.getClass() != CPrototype.class) {
	    listeElementsAncetres.retireTout();
	    int indicePointATracer = listeProprietaire.indexOf(pointATracer);
	    int indicePointLieGenerateur = listeProprietaire.indexOf(pointLieGenerateur);
	    listeElementsAncetres.add(pointLieGenerateur);
      for (int i = indicePointLieGenerateur + 1; i < indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
      listeElementsAncetres.add(pointATracer);
    }	
  }
  
	/**
	 * Fonction renvoyant true si l'objet dpend des positions gnres pas les dplacements d'un point li
	 * A redfinir pour les lieux de points et lieux d'objets et macros utilisant les dplacements d'un point li
	 * @param pointlie
	 * @return
	 */
  @Override
	public boolean estGenereParPointLie(CPointLie pointlie) {
		return pointLieGenerateur == pointlie;
	}
  
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list,nVersion >= 2);
    int ind1 = inps.readCInt();
    pointLieGenerateur = (CPointLie) (list.get(ind1,CPointLie.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    int indicePointATracer = list.indexOf(pointATracer);
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
    }
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    pointLieGenerateur = (CPointLie) (list.get(ind1,CPointLie.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modifi version 3.4.2
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    /*
    int indicePointATracer = list.indexOf(pointATracer);
    if (list.getClass() != CPrototype.class) {
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
    }
    */
    if (nVersion == 1) etablitListeElementsAncetres();
    else listeElementsAncetres.read(inps, list);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointLieGenerateur);
    oups.writeInt(ind1);
    listeElementsAncetres.write(oups, listeProprietaire);
  }     
}
