/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CLieuDiscretParVariable extends CLieuDiscretDeBase {
  CVariableBornee variableGeneratrice;
  
  /** Creates new CLieuDiscretParVariable */
  public CLieuDiscretParVariable() {
  }
  
  // Constructeur pour clonage
  public CLieuDiscretParVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, CPointAncetre pointATracerInit, int nombreDePointsInit, int motifInit,
    CVariableBornee variableGeneratriceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit,
      masqueInit, pointATracerInit, nombreDePointsInit, motifInit);
    variableGeneratrice = variableGeneratriceInit;
    // Rajout version 3.4.2
    // etablitListeElementsAncetres(); Inutile et provoquait un bug sous version 4.4 en appelant getClone
  }
  
  /**
   * A partir de la version 3.4.2, changement de version (version 1 -> 2) car on enregistre dans le flux la CSousListeObjetstres
   * contenant les pointeurs sur les lments anctres
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointATracer);
    int ind2 = listeSource.indexOf(variableGeneratrice);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CLieuDiscretParVariable(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      nombreDePoints, motif, (CVariableBornee)(listeCible.get(ind2, CVariableBornee.class)));
    if (listeCible.getClass() != CPrototype.class) {
	    // Supprim version 3.4.2: Inutile ((CLieuDiscretParVariable) ptelb).listeElementsAncetres = new CListeObjets(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
	    // Bug corrig version 4.7.6
    	// listeElementsAncetres.setImage(((CLieuDiscretParVariable)ptelb).listeElementsAncetres, ptelb.listeProprietaire,
	    //  listeProprietaire);
	    ((CLieuDiscretParVariable)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire, ptelb.listeProprietaire);
    }
    return ptelb;
  }
  
  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   */
  @Override
  public void metAJour() {
  	super.metAJour();
  	// Bug corrig version 4.0 : il faut mettre  jour la liste d'lments anctres car metAJour est appel quand
  	// on modifie un objet dont dpend cet objet
  	// if (nVersion == 1) etablitListeElementsAncetres();
  	etablitListeElementsAncetres();
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double valeur, valeurMinimale, valeurMaximale, valeurInitiale, pas;
    
    if (! variableGeneratrice.existe) {
      existe = false;
      return;
    }
    
    valeurMinimale = variableGeneratrice.valeurMini;
    valeurMaximale = variableGeneratrice.valeurMaxi;
    valeurInitiale = variableGeneratrice.valeurActuelle;
    pas = (valeurMaximale-valeurMinimale)/(nombreDePoints-1);
    valeur = valeurMinimale;
    indiceDernierPointTrace = -1;
    int nb = nombreDePoints - 1;
    for (int i = 0; i < nombreDePoints; i++) {
      valeur = (i == 0) ? valeurMinimale :((i == nb) ? valeurMaximale : valeur + pas);
      variableGeneratrice.donneValeur(valeur);
      listeElementsAncetres.positionne(infoRandom, dimfen);
      if (pointATracer.existe) {
        if (Kernel.testToile(pointATracer.x,pointATracer.y)) {
          indiceDernierPointTrace++;
          abscissesCoordonneesPoints[indiceDernierPointTrace] = (int) Math.round(pointATracer.x);
          ordonneesCoordonneesPoints[indiceDernierPointTrace] = (int) Math.round(pointATracer.y);
        }
      }
    }
    
    // On redonne  la variable gnratrice sa valeur initiale
    variableGeneratrice.donneValeur(valeurInitiale);
    listeElementsAncetres.positionne(infoRandom, dimfen);
    if (indiceDernierPointTrace >= 0)
      existe = true;
    else existe = false;
  }
  
  public boolean confonduAvec(CElementBase p) {
    CLieuDiscretParVariable pt;
    if (p.getClass() == this.getClass()) {
      pt = (CLieuDiscretParVariable) p;
      return (pointATracer == pt.pointATracer) && (variableGeneratrice == pt.variableGeneratrice);
    } else return false;
  }
    
  public void etablitListeElementsAncetres() {
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (listeProprietaire.getClass() != CPrototype.class) {
	    listeElementsAncetres.retireTout();
	    int indicePointATracer = listeProprietaire.indexOf(pointATracer);
	    int indiceVariableGeneratrice = listeProprietaire.indexOf(variableGeneratrice);
	    listeElementsAncetres.add(variableGeneratrice);
      for (int i = indiceVariableGeneratrice + 1; i < indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
      listeElementsAncetres.add(pointATracer);
    }	
  }

  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    variableGeneratrice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      int indicePointATracer = list.indexOf(pointATracer);
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
    }
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    variableGeneratrice = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    // Modifi version 3.4.2
    /*
    if (list.getClass() != CPrototype.class) {
      int indicePointATracer = list.indexOf(pointATracer);
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(variableGeneratrice)))
          listeElementsAncetres.add(ptelb);
      }
    }
    */
    if (nVersion == 1) etablitListeElementsAncetres();
    else listeElementsAncetres.read(inps, list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(variableGeneratrice);
    oups.writeInt(ind1);
    listeElementsAncetres.write(oups, listeProprietaire);
  }     
}
