/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import java.util.*;
import java.awt.Color;
import Applet.AppletMathGraph32;
import Applet.MtgPaneFigure;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


public class CMacroBoucleAvecAnimation extends CMacro {
  public int frequenceAnimation;
	public boolean animationVisible;
	public CVariableBornee variableAssociee;
	public CMacro macroAvantBoucles;
	public CMacro macroFinBoucle;
  private double pas;
  private double valeurMinimale;
  private double valeurMaximale;
	private double valeurCourante;
	private CListeObjets listeElementsDependants;
	private CListeObjets listeElementsDependantMacroFin;
	private CListeObjets listeVariablesModifiees;

  /** Creates new CMacroBoucleAvecAnimation */
  public CMacroBoucleAvecAnimation() {
  }
  
  // Constructeur pour clonage
  public CMacroBoucleAvecAnimation(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      int frequenceAnimationInit, boolean animationVisibleInit, CVariableBornee variableAssocieeInit,
      CMacro macroAvantBouclesInit, CMacro macroFinBoucleInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
					alignementHorizontalInit , alignementVerticalInit);
      frequenceAnimation = frequenceAnimationInit;
      animationVisible = animationVisibleInit;
      variableAssociee = variableAssocieeInit;
      macroAvantBoucles = macroAvantBouclesInit;
      macroFinBoucle = macroFinBoucleInit;
      listeElementsDependants = new CListeObjets(liste);
      listeElementsDependantMacroFin = new CListeObjets(liste);
      listeVariablesModifiees = new CListeObjets(liste);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CMacro mac2, mac3;
    int ind1 = listeSource.indexOf(variableAssociee);
    if (macroAvantBoucles == null) mac2 = null;
    else {
      int ind2 = listeSource.indexOf(macroAvantBoucles);
      mac2 = (CMacro) (listeCible.get(ind2, CMacro.class));
    }
    if (macroFinBoucle == null) mac3 = null;
    else {
      int ind3 = listeSource.indexOf(macroFinBoucle);
      mac3 = (CMacro) (listeCible.get(ind3, CMacro.class));
    }
    int ind4 = listeSource.indexOf(pointLie);
    int ind5 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroBoucleAvecAnimation(listeCible, (CImplementationProto)(listeCible.get(ind5, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind4, CPointAncetre.class), indiceTaillePolice, 
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, frequenceAnimation, animationVisible,
      (CVariableBornee)(listeCible.get(ind1, CVariableBornee.class)), mac2, mac3,
      alignementHorizontal, alignementVertical);
    ((CMacroBoucleAvecAnimation)ptelb).listeElementsDependants = new CListeObjets(listeCible);
    ((CMacroBoucleAvecAnimation)ptelb).listeElementsDependantMacroFin = new CListeObjets(listeCible);
    ((CMacroBoucleAvecAnimation)ptelb).listeVariablesModifiees = new CListeObjets(listeCible);
    return ptelb;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    boolean resultat = false;
    if (macroAvantBoucles != null)
      resultat |= macroAvantBoucles.dependDe(p);
    if (macroFinBoucle != null)
      resultat |= macroFinBoucle.dependDe(p);
    return memoriseDependance(resultat || super.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    boolean resultat = false;
    if (macroAvantBoucles != null)
      resultat |= macroAvantBoucles.dependDePourBoucle(p);
    if (macroFinBoucle != null)
      resultat |= macroFinBoucle.dependDePourBoucle(p);
    return resultat || super.dependDePourBoucle(p);
  }

  @Override
  public int frequenceAnimation() {
    return frequenceAnimation;
  }
  
  @Override
  public void metAJour() {
  	super.metAJour();
  	etablitListesInternes();
  }
  
  /**
   * Fonction prparant les listes dont la macro a besoin pour s'excuter
   * Est appel aprs le chargement complet de la liste car des lments peuvent dpendre de la longueur unit
   */
  @Override
  public void etablitListesInternes() {
  	listeElementsDependants.retireTout();
  	listeVariablesModifiees.retireTout();
  	listeElementsDependantMacroFin.retireTout();
    ListIterator<CElementBase> li1 = listeProprietaire.listIterator();
    while(li1.hasNext()) {
      CElementBase ptb1 = li1.next();
      if (ptb1.dependDePourBoucle(variableAssociee))
      {
        listeElementsDependants.add(ptb1);
      }
    }
    if (macroFinBoucle != null)
    {
      ListIterator<CElementBase> li2 = listeProprietaire.listIterator();
      while(li2.hasNext()) {
        CElementBase ptb1 = li2.next();
        if (ptb1.getNatureCalcul() == NatureCalcul.NVariable)
        {
          CVariableBornee ptv1 = (CVariableBornee) ptb1;
          if ((ptv1 != variableAssociee) && macroFinBoucle.agitSurVariable(ptv1))
            listeVariablesModifiees.add(ptb1);
        }
      }
      // Il faut aussi crer la liste des lments dpendants
      // d'une variable sur laquelle agit la macro
      // excute  la fin de chaque liste
      ListIterator<CElementBase> li3 = listeProprietaire.listIterator();
      while(li3.hasNext()) {
        CElementBase ptb1 = li3.next();
        if (ptb1.getNature() != NatureObjet.NMacro)
        {
          ListIterator<CElementBase> li4 = listeVariablesModifiees.listIterator();
          while(li4.hasNext())
          {
            CVariableBornee ptv2 = (CVariableBornee) (li4.next());
            if (ptb1.dependDe(ptv2)) {
              if (!listeElementsDependantMacroFin.contains(ptb1)) {
              	if ((ptb1.getNatureCalcul() & NatureCalcul.NTouteValeurReelleSaufConst) != 0) {
              		if (!macroFinBoucle.affecteValeurAVariable((CValeurDynamique) ptb1))
              			listeElementsDependantMacroFin.add(ptb1);
              	}
              	else listeElementsDependantMacroFin.add(ptb1);
              }
            }
          }
        }
      }
    }    	
  }
  
  @Override
  public void executePourTimer(AppletMathGraph32 applet, boolean repaint) {
    if (!executionEnCours) return;
    MtgPaneFigure paneFigure = applet.paneFigure;
		// listeGraphiqueElementsDependants.effaceTout(paneFigure.gbuffer,applet.dimf, true);
		variableAssociee.donneValeur(valeurCourante);
		// Gnrera des valeurs alatoires dans les appels de Rand()
		listeElementsDependants.positionne(applet.infRand, applet.dimf);
		// On excute la macro de fin de boucle
		if ((macroFinBoucle != null) && (valeurCourante <= valeurMaximale))
		{
			if (macroFinBoucle.executionPossible()) macroFinBoucle.executeSimple(applet);
			listeElementsDependantMacroFin.positionne(applet.infRand, applet.dimf);
		}
    if (repaint) paneFigure.repaint();
    valeurCourante += pas;
    if (valeurCourante > valeurMaximale) {
    	termineAction(applet);
      return;
    }
		Thread.yield(); // Pour viter les ConcurrentModificationException
  } 
  
  @Override
  public void actionAvantExecution(AppletMathGraph32 applet) {
  	super.actionAvantExecution(applet);  // Met executioEnCours  true
    applet.setBackground(applet.couleurFond);
    Couleur.setCouleurFond(applet.couleurFond);
    if (macroAvantBoucles != null)
    {
      if (macroAvantBoucles.executionPossible()) macroAvantBoucles.executeSimple(applet);
    }
    //  revoir
    pas = variableAssociee.valeurPas;
    valeurMinimale = variableAssociee.valeurMini;
    valeurMaximale = variableAssociee.valeurMaxi;
    // Ligne suivante modifie version 2.5.7
    // valeurCourante = valeurMinimale - pas;
    valeurCourante = valeurMinimale;
    // On permet le tirage alatoire par la fonction Rand()
    applet.infRand = true;
  }
  
  
	// Modification version 2.0. Etait vide avant.
  @Override
  public void actionDansPaint(AppletMathGraph32 applet) {
  	if (animationVisible) {
	  	applet.paneFigure.effacegBuffer();
	    applet.paneFigure.afficheToutDansgbuffer();
  	}
  }

  @Override
  public void termineAction(AppletMathGraph32 applet) {
    super.termineAction(applet);
    MtgPaneFigure paneFigure = applet.paneFigure;
    // On ne permet plus le tirage alatoire par la fonction Rand()
    applet.infRand = false;
		paneFigure.effaceFenetre(paneFigure.gbuffer);
    paneFigure.afficheToutDansgbuffer();
    applet.paneFigure.repaint();
  }
	// Ajout version 4.7.7 . On peut vouloir tracer laisser les traces pour un seul objet qui peut ne pas exister
	// au dbut.
  @Override
  public boolean executionPossible() {
  	return true;
  }
  @Override
  public void executePourThread(AppletMathGraph32 applet) {
    actionAvantExecution(applet);
    MtgPaneFigure paneFigure = applet.paneFigure;
    // Il est ncessaire d'ausculter executionEnCours pour que la boucle ne continue
    // pas alors qu'on a appuy sur la touche Escape
    while ((valeurCourante <= valeurMaximale) && executionEnCours) {
      variableAssociee.donneValeur(valeurCourante);
      // Gnrera des valeurs alatoires dans les appels de Rand()
      listeElementsDependants.positionne(applet.infRand, applet.dimf);
      // On excute la macro de fin de boucle
      // La macro de fin de boucle doit tre effectue sans recalcul
      // ni raffichage de la figure
      if ((macroFinBoucle != null) && (valeurCourante <= valeurMaximale)) {
        if (macroFinBoucle.executionPossible()) macroFinBoucle.executeSimple(applet);
        listeElementsDependantMacroFin.positionne(applet.infRand, applet.dimf);
      }
      if (applet.runner != null) Thread.yield();
      valeurCourante += pas;
      if (animationVisible) {
      	paneFigure.repaint();
      }
    }
    // executionEnCours = false;
    termineAction(applet); // Pour remettre les listes  0
    applet.runner = null;
  }
  
  @Override
  public boolean agitSurVariable(CVariableBornee var) {
    return (macroAvantBoucles.agitSurVariable(var) || macroFinBoucle.agitSurVariable(var));
  }

  @Override
  public int typeAnimation() {
    if ((animationVisible)&& (frequenceAnimation != 0)) return CMacro.animationParTimer;
    else return CMacro.animationParThread;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    animationVisible = inps.readBoolean();
		frequenceAnimation = inps.readCInt();
    // Frquence  vitesse maxi remplace par 1/1000 de seconde pour l'applet
    // if (frequenceAnimation == 0) frequenceAnimation = 1;
    int ind1 = inps.readCInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    int ind2 = inps.readCInt();
    if (ind2 == -1) macroAvantBoucles = null;
    else macroAvantBoucles = (CMacro) (list.get(ind2,CMacro.class));
    int ind3 = inps.readCInt();
    if (ind3 == -1) macroFinBoucle = null;
    else macroFinBoucle = (CMacro) (list.get(ind3,CMacro.class));  
    if (list.getClass() != CPrototype.class) {
      listeElementsDependants = new CListeObjets(listeProprietaire);
      listeElementsDependantMacroFin = new CListeObjets(listeProprietaire);
      listeVariablesModifiees = new CListeObjets(listeProprietaire);
    }
  }
  /* Lecture depuis dans un flux java */
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    animationVisible = inps.readBoolean();
		frequenceAnimation = inps.readInt();
    int ind1 = inps.readInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    int ind2 = inps.readInt();
    if (ind2 == -1) macroAvantBoucles = null;
    else macroAvantBoucles = (CMacro) (list.get(ind2,CMacro.class));
    int ind3 = inps.readInt();
    if (ind3 == -1) macroFinBoucle = null;
    else macroFinBoucle = (CMacro) (list.get(ind3,CMacro.class));  
    if (list.getClass() != CPrototype.class) {
      listeElementsDependants = new CListeObjets(listeProprietaire);
      listeElementsDependantMacroFin = new CListeObjets(listeProprietaire);
      listeVariablesModifiees = new CListeObjets(listeProprietaire);
    }
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeBoolean(animationVisible);
    oups.writeInt(frequenceAnimation);
    int ind1 = list.indexOf(variableAssociee);
    oups.writeInt(ind1);
    int ind2;
    if (macroAvantBoucles == null) ind2 = -1;
    else ind2 = list.indexOf(macroAvantBoucles);
    oups.writeInt(ind2);
    int ind3;
    if (macroFinBoucle == null) ind3 = -1;
    else ind3 = list.indexOf(macroFinBoucle);
    oups.writeInt(ind3);
  }
}
