/*
 * MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Ajout version 4.8
import java.awt.Color;
import java.io.IOException;
import java.util.Vector;
import outils.Outil;
import Applet.AppletMathGraph32;
import types.NatureObjet;
import types.StyleRemplissage;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CMacroConstructionRecursive extends CMacroAvecListe {
	public Vector <CPrototype> tableProto;
	public int profondMax = 2; // La profondeur de rcursion
	// public CPrototype proto; // La construction qui sera excute par la macro
	public int pasRec = 1; // Le pas de rcursion
	public int nbImp = 4; // Le nombre d'appels rcursifs
	public int nbSourcesPourIter; // Le nombre d'objets sources communs  chaque itration
	public boolean creationNiveauMax; // Quand false, seuls les objets finaux servant  l'implmentation suivant sont crs
	public boolean imposerCouleur; // Si on souhaite donner la mme couleur  tous les objets finaux graphiques crs
	public Color couleurImp; // La couleur des objets crs dans le cas o imposerCouleur est true
	public int choixLigne; // Choix de joindre les points crs par rien, une ligne brise ou un polygone
	public int styleLigne; // Dans le cas d'une ligne, style de trait de la ligne
	public Color couleurLigne; //  Dans le cas d'une ligne, style de trait de la ligne
	public boolean polygoneRempli = false; // Dans le cas o un polygone est demand, cre la surface qu'il dlimite
	private boolean dejaExecute = false;

	/**
	 * Constructeur par dfaut
	 */
	public CMacroConstructionRecursive() {
		dejaExecute = false;
	}
	
	// Constructeur pour clonage
	public CMacroConstructionRecursive(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
		  Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
		  CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
		  boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
		  CSousListeObjets listeAssocieeInit, int alignementHorizontalInit, int alignementVerticalInit, int profondMaxInit,
		  Vector<CPrototype> tableProtoInit, int pasRecInit, int nbImpInit, int nbSourcesPourIterInit, boolean creationNiveauMaxInit,
		  boolean imposerCouleurInit, Color couleurImpInit, int choixLigneInit, int styleLigneInit, Color couleurLigneInit, boolean polygoneRempliInit) {
		  super(liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
		    dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
		    effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, listeAssocieeInit,
		    alignementHorizontalInit, alignementVerticalInit);
		  //proto = protoInit;
		  tableProto = tableProtoInit;
		  pasRec = pasRecInit;
		  nbImp = nbImpInit;
		  profondMax = profondMaxInit;
		  nbSourcesPourIter = nbSourcesPourIterInit;
		  creationNiveauMax = creationNiveauMaxInit;
		  imposerCouleur = imposerCouleurInit;
		  couleurImp = couleurImpInit;
		  choixLigne = choixLigneInit;
		  styleLigne = styleLigneInit;
		  couleurLigne = couleurLigneInit;
		  polygoneRempli = polygoneRempliInit;
			dejaExecute = false;
		}
		
		@Override
		public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		  int ind1 = listeSource.indexOf(pointLie);
		  int ind2 = listeSource.indexOf(impProto);
		  CSousListeObjets listeAssocieeClone = new CSousListeObjets();
		  listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
		  CElementBase ptelb = new CMacroConstructionRecursive(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
		    estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
		    (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
		    effacementFond, couleurFondAffichage, intitule, commentaireMacro, listeAssocieeClone,
		    alignementHorizontal, alignementVertical, profondMax, tableProto, pasRec, nbImp, nbSourcesPourIter, creationNiveauMax,
		    imposerCouleur, couleurImp, choixLigne, styleLigne, couleurLigne, polygoneRempli);
		  return ptelb;
		}
	
  /**
   * Ajout version 4.8 : Renvoie true si la macro est une CMacroConstructionIterative ou une CMacroConstructionRecursive
   * ou une CMacroLSuiteMacros ne contenant que des macros du type prcdent.
   * Redfini pour CMacroConstructionIteartive, CMacroConstructionRecursive et CMacroSuiteMacros
   * @return
   */
  public boolean utilisablePourDemarrage() {
  	return true;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("macroConstructionRecursiveDlg1");
  }

  /**
   * Si la figure comporte une longuer unit et si la construction associe utilise la longueur unit, dependDe doit rendre true
   */
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	boolean b = super.dependDe(p);
  	if (p == listeProprietaire.pointeurLongueurUnite) {
  		for (int i = 0; i <= profondMax; i++) b = b || tableProto.get(i).utiliseLongueurUnite();
  		return  memoriseDependance(b) ;
  	}
  	else return memoriseDependance(b);
  }

	@Override
	public boolean dependDeProto(CPrototype proto) {
		for (int i = 0; i <= profondMax; i++) {
			CPrototype prot = tableProto.get(i);
			if (prot == proto) return true;
		}
		return false;
	}
	
	@Override
	public boolean executionPossible() {
		// Spcial Applet
		return !dejaExecute;
	}
	
  @Override
  public void actionAvantExecution(AppletMathGraph32 cadre) { // par dfaut, met executionEnCours  true;
  	super.actionAvantExecution(cadre);  // Met executionEnCours  true
		CPrototype proto0 = tableProto.get(0);
		// On implmente la construction de faon itrative
  	// La liste associe contient les pointeurs sur les objets sources
  	// On affecte ces lments aux objets sources du prototype
  	for (int j = 0; j <= profondMax; j++) {
  		CPrototype proto = tableProto.get(j);
	  	for (int i = 0; i < proto.nbObjSources; i++) ((CElementGenerique)proto.get(i)).elementAssocie = listeAssociee.get(i);
  	}
		CListeObjets liste = cadre.listePr;
    Vector<CNoeudPointeurSurPoint> listePoints = choixLigne == 0 ? null
    		: new Vector<CNoeudPointeurSurPoint> (); // Pour joindre ventuellement les points de dernire gnration
		CImplementationProto impProto = new CImplementationProto(liste, proto0);
		// On cre une liste forme des objets sources fixes et une autre forme des objets sources variables.
		CListeObjets listeObjetsSourcesConst = new CListeObjets();
		for (int i = 0; i < nbSourcesPourIter; i++) {
			CElementGenerique eg = (CElementGenerique)(proto0.get(i));
			listeObjetsSourcesConst.add(eg.elementAssocie);
		}
		CListeObjets listeObjetsSourcesVar = new CListeObjets();
		for (int i = nbSourcesPourIter; i < proto0.nbObjSources; i++) {
			CElementGenerique gen = (CElementGenerique) proto0.get(i);
			listeObjetsSourcesVar.add(gen.elementAssocie);
		}
		int indprot = liste.size();
		int profond = 0;
		impProto.implementeRecursif(liste, tableProto, cadre, listeObjetsSourcesConst, listeObjetsSourcesVar, pasRec, nbImp,
				creationNiveauMax, profond, profondMax, listePoints);
		liste.traiteImplementationsProtoSuccessivesRecur(indprot, proto0, cadre);
		// liste.reconstruitChainesCalcul(indprot + 1, indprot + proto0.size() - proto0.nbObjSources);
		if (imposerCouleur) {
	    for (int i= liste.indexOf(impProto)+1; i < liste.size(); i++) {
	    	CElementBase elb = liste.get(i);
	    	if (elb.estElementFinal && (elb.estDeNature(NatureObjet.NToutObjet))) {
	    		CElementGraphique elg = (CElementGraphique) elb;
	    		elg.donneCouleur(couleurImp);
	    	}
	    }
    }
		/*
		if (unite != null) {
    	// Si on a accept l'implmentation alors que la figure ne contient pas de longueur unit c'est que
    	// le premier objet utilisant une longueur dans le prototype est une longueur et est un objet final
    	// Ce sera en gnral une longueur unit
			liste.pointeurLongueurUnite = liste.premiereLongueur();
		}
		*/
		// On regarde si on moins un des objets finaux crs existe
    // Si demand un polygone ou une ligne on joint tous les points crs.
    if (choixLigne != 0) {
	    if (choixLigne == 1) { // Polygone
				listePoints.add(new CNoeudPointeurSurPoint(liste, listePoints.firstElement().pointeurSurPoint));
				CPolygone poly = new CPolygone(liste, listePoints, couleurLigne, styleLigne);
				liste.add(poly);
				poly.positionne(false, cadre.dimf);
				if (polygoneRempli) {
					CSurfacePolygone s = new CSurfacePolygone(liste, null, false, couleurLigne, false, StyleRemplissage.QuadrillagePleinTransp, poly);
					liste.add(s);
					s.positionne(false, cadre.dimf);
				}
	    }
	    else {
				CLigneBrisee ligne = new CLigneBrisee(liste, listePoints, couleurLigne, styleLigne);
				liste.add(ligne);
				ligne.positionne(false, cadre.dimf);
	    }
    }
    
		// On sauvegarde l'tat de la figure
		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil115"));
		// Version 4.5 : Inutile de recalculer la figure car cela a dj t fait dans CImplementationProoto.implemente
	  // cadre.repaintFigureAvecCalcul();
		cadre.repaintFigureSansCalcul();
		dejaExecute = true;
  }
 

  @Override
  public void actionDansPaint(AppletMathGraph32 applet) {
  	termineAction(applet);
  }

  @Override
  public boolean modifiableParMenu() {
  	return !estElementFinal;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    profondMax = inps.readInt();
    tableProto = new Vector<CPrototype>();
    for (int i = 0; i <= profondMax; i++) {
    	int indProto = inps.readInt();
    	tableProto.add(list.documentProprietaire.tablePrototypes.get(indProto));
    }
    pasRec = inps.readInt();
    nbImp = inps.readInt();
    nbSourcesPourIter = inps.readInt();
    creationNiveauMax = inps.readBoolean();
    imposerCouleur = inps.readBoolean();
    if (imposerCouleur) {
  		int r = inps.readUnsignedByte();
  		int g = inps.readUnsignedByte();
  		int b = inps.readUnsignedByte();
  		couleurImp = new Color(r, g, b, 255);
    }
    choixLigne = inps.readInt();
    if (choixLigne != 0) {
  		int r = inps.readUnsignedByte();
  		int g = inps.readUnsignedByte();
  		int b = inps.readUnsignedByte();
  		couleurLigne = new Color(r, g, b, 255);
    	styleLigne = inps.readInt();
    	if (choixLigne == Outil.avecPolygone)
    		polygoneRempli = inps.readBoolean();
    }
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(profondMax);
    // On enregistre l'indice du prototype utilis dans la table des prototype du document
    for (int i = 0; i <= profondMax; i++) oups.writeInt(list.documentProprietaire.tablePrototypes.indexOf(tableProto.get(i)));
    oups.writeInt(pasRec);
    oups.writeInt(nbImp);
    oups.writeInt(nbSourcesPourIter);
    oups.writeBoolean(creationNiveauMax);
    oups.writeBoolean(imposerCouleur);
    if (imposerCouleur) {
  		oups.writeByte(couleurImp.getRed());
  		oups.writeByte(couleurImp.getGreen());
  		oups.writeByte(couleurImp.getBlue());
    }
    oups.writeInt(choixLigne);
    if (choixLigne != 0) {
  		oups.writeByte(couleurLigne.getRed());
  		oups.writeByte(couleurLigne.getGreen());
  		oups.writeByte(couleurLigne.getBlue());
    	oups.writeInt(styleLigne);
    	if (choixLigne == Outil.avecPolygone)
    		oups.writeBoolean(polygoneRempli);
    }
  }  

}
