/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CSolutionEquation extends CCalculAncetre {
	// Importante modification de la version 4.3 : On entre un calcul d'une variable qui dfinira l'quation
  // public CFonc fonctionAssociee;
	public CCalculBase equation; // Le calcul reprsentant l'quation avec un test d'galit
	public String inconnue; // Chane de caractres reprsentant l'inconnue de l'quation
	public CCalculBase fonction;		// Un calcul reprsentant une fonction d'une variable. La solution sera celle de f(x) = 0
  // Les bornes de l'intervalle de recherche
  public CValeurAssocieeAVariable a;
  public CValeurAssocieeAVariable b;
  // L'incertitude recherche
  public CValeurAssocieeAVariable incertitude;
  // La valeur de la solution de l'quation
  private double solution;
  
  /** Creates new CSolutionEquation */
  public CSolutionEquation() {
  }
  
  // Pour la version 4.3 on change la nature de l'objet qui devient une quation quelconque
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  /**
   * Constructeur pour bote de dialogue
   * @param liste
   * @param impProtoInit
   * @param estElementFinalInit
   * @param nomInit
   * @param fonctionAssocieeInit
   * @param aInit
   * @param bInit
   * @param incertitudeInit
   */
  public CSolutionEquation(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CValeurAssocieeAVariable aInit, CValeurAssocieeAVariable bInit, CValeurAssocieeAVariable incertitudeInit) {
      super(liste, impProtoInit, estElementFinalInit, nomInit);
      a = aInit;
      b = bInit;
      incertitude = incertitudeInit;
      inconnue = "x";
      equation = null;
      fonction = null;
  }
  
  // Constructeur pour clonage
  public CSolutionEquation(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomInit, CValeurAssocieeAVariable aInit, CValeurAssocieeAVariable bInit,
    CValeurAssocieeAVariable incertitudeInit, String inconnueInit, CCalculBase equationInit, CCalculBase fonctionInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    a = aInit;
    b = bInit;
    incertitude = incertitudeInit;
    inconnue = inconnueInit;
    equation = equationInit;
    fonction = fonctionInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable aClone = (CValeurAssocieeAVariable) a.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable bClone = (CValeurAssocieeAVariable) b.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable incertitudeClone = (CValeurAssocieeAVariable) incertitude.getClone(listeSource, listeCible);
    CCalculBase equationClone = equation.getClone(listeSource, listeCible);
    CCalculBase fonctionClone = fonction.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSolutionEquation(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, aClone, bClone, incertitudeClone, inconnue, equationClone, fonctionClone);
    return ptelb;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NSolutionEquation;
  }
  
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	a.initialisePourDependance();
  	b.initialisePourDependance();
  	incertitude.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || a.dependDe(p) || b.dependDe(p)
    || incertitude.dependDe(p) || equation.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || a.dependDePourBoucle(p) || b.dependDePourBoucle(p)
    || incertitude.dependDePourBoucle(p) || equation.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double xm,xa,xb,ya,yb,ym,e;
    a.positionne(infoRandom, dimfen);
    b.positionne(infoRandom, dimfen);
    incertitude.positionne(infoRandom, dimfen);
    existe = equation.existe(infoRandom) && a.existe() && b.existe() && incertitude.existe() ;
    if (!existe) return;
    xa = a.rendValeur(infoRandom);
    xb = b.rendValeur(infoRandom);
    e = incertitude.rendValeur(infoRandom);
    if (xa > xb) {
      double temp = xa;
      xa = xb;
      xb = temp;
    }
    try {
      ya = fonction.resultatFonction(infoRandom, xa);
      yb = fonction.resultatFonction(infoRandom, xb);
      if ((ya*yb > 0) || (e <= 0)) {
        existe = false;
        return;
      }
      
    } catch (ErreurCalculException err) {
      existe = false;
      return;
    }
    double nBoucles = Math.floor(Math.log(Math.abs(xb - xa)/e)/Math.log(2)) + 1;
    if (nBoucles > 1000) {
      existe = false;
      return;
    }
    xm = (xa + xb) /2;
    for (int i = 1; i <= (int) nBoucles; i ++) {
      xm = (xa + xb) /2;
      try {
        ym = fonction.resultatFonction(infoRandom,xm);
      } catch (ErreurCalculException err) {
        existe = false;
        return;
      }
      if (ym*ya > 0) xa = xm;
      else xb = xm;
    }
    existe = true;
    solution = xm;
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return solution;
  }
  
  @Override
  public String infoPourHistorique() {
  	String[] var = new String[1];
  	var[0] = inconnue;
    String ch = getString("chinfo98") + " " + nom() + " " + getString("chinfo99") + " " + equation.chaineCalculSansPar(var) + "\n" +
    getString("chinfo100") + " " + a.chaineInfo() + " " + getString("chinfo101") + " " + b.chaineInfo() + "\n" +
    getString("chinfo102") + " " + incertitude.chaineInfo();
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + glob.Kernel.chaineNombre(solution, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
      
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readInt();
    CFonc fonc = (CFonc) (list.get(ind1,CFonc.class));
    fonction = fonc.calcul;
    inconnue = fonc.chaineVariable;
    equation = new COperation(list, new CAppelFonction(list, fonc, new CVariableFormelle(list, 0)), 
    		new CConstante(list,0), Operateur.Egalite);
    a = new CValeurAssocieeAVariable();
    a.chargeDepuisFlux(inps, list);
    b = new CValeurAssocieeAVariable();
    b.chargeDepuisFlux(inps, list);
    incertitude = new CValeurAssocieeAVariable();
    incertitude.chargeDepuisFlux(inps, list);
  }

   @Override
   public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    if (nVersion < 2) {
	    int ind1 = inps.readInt();
	    CFonc fonc = (CFonc) (list.get(ind1,CFonc.class));
	    fonction = fonc.calcul;
	    inconnue = fonc.chaineVariable;
	    equation = new COperation(list, new CAppelFonction(list, fonc, new CVariableFormelle(list, 0)), 
	    		new CConstante(list,0), Operateur.Egalite);
    }
    else {
    	inconnue = inps.readUTF();
      equation = (CCalculBase) (inps.readObject(list));
      fonction = (CCalculBase) (inps.readObject(list));    	
    }
    a = new CValeurAssocieeAVariable();
    a.read(inps, list);
    b = new CValeurAssocieeAVariable();
    b.read(inps, list);
    incertitude = new CValeurAssocieeAVariable();
    incertitude.read(inps, list);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(inconnue);
    oups.writeObject(equation);
    oups.writeObject(fonction);
    a.write(oups, list);
    b.write(oups, list);
    incertitude.write(oups, list);
  }
  
  @Override
	public boolean estConstant() {
		return a.estConstant() && b.estConstant() && incertitude.estConstant() && equation.estConstant();
	}

}
