/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;

import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CSurfaceDeuxLieux extends CSurface {
	CLieuDeBase deuxiemeLieu;

	private int nombrePointsTraces; // Nombre de points effectifs tracs
	private int[] abscisses;
	private int[] ordonnees;
	private CPointBase pointDebutLieu1;
	private CPointBase pointDebutLieu2;
	private CPointBase pointFinLieu1;
	private CPointBase pointFinLieu2;
	private CSegment segmentDebut;
	private CSegment segmentFin;

	/** Creates new CSurfaceDeuxLieux */
	public CSurfaceDeuxLieux() {
	}

	// Constructeur pour clonage
	public CSurfaceDeuxLieux(CListeObjets liste,
			CImplementationProto impProtoInit, boolean estElementFinalInit,
			Color couleurInit, boolean masqueInit, int styleRemplissageInit,
			CElementGraphique bordInit, CLieuDeBase deuxiemeLieuInit) {
		super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit,
				styleRemplissageInit, bordInit);
		deuxiemeLieu = deuxiemeLieuInit;
		pointDebutLieu1 = new CPointBase();
		pointDebutLieu2 = new CPointBase();
		pointFinLieu1 = new CPointBase();
		pointFinLieu2 = new CPointBase();
		segmentDebut = new CSegment(liste, pointDebutLieu1, pointDebutLieu2);
		segmentFin = new CSegment(liste, pointFinLieu1, pointFinLieu2);
		// On cre les tableaux devat recevoir les coordonnes. Le + 3 est du aux
		// deux points projets
		// sur la droite
		// Modifi pour la version 1.9.5. Seules deux coordonnes sont rajoutes car
		// il est
		// inutile de refermer le polygone
		// abscisses = new int
		// [((CLieuDeBase)bord).infoLieu.nombreDePoints +
		// deuxiemeLieu.infoLieu.nombreDePoints + 1];
		// ordonnees = new int
		// [((CLieuDeBase)bord).infoLieu.nombreDePoints +
		// deuxiemeLieu.infoLieu.nombreDePoints + 1];
		metAJourTableaux();
	}

  @Override
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		int ind1 = listeSource.indexOf(bord);
		int ind2 = listeSource.indexOf(deuxiemeLieu);
		int ind3 = listeSource.indexOf(impProto);
		return new CSurfaceDeuxLieux(listeCible, (CImplementationProto) listeCible.get(ind3, CImplementationProto.class),
				estElementFinal, couleur, masque, styleRemplissage,
				(CLieuDeBase) (listeCible.get(ind1, CLieuDeBase.class)),
				(CLieuDeBase) (listeCible.get(ind2, CLieuDeBase.class)));
	}

  @Override
	public void setClone(CElementBase ptel) {
		super.setClone(ptel);
		if (ptel.existe) {
			nombrePointsTraces = ((CSurfaceDeuxLieux) ptel).nombrePointsTraces;
			// Modifi pour la version 1.9.5
			// int nbPointsADecaler = nombrePointsTraces + 1;
			System.arraycopy(((CSurfaceDeuxLieux) ptel).abscisses, 0, abscisses, 0,
					nombrePointsTraces);
			System.arraycopy(((CSurfaceDeuxLieux) ptel).ordonnees, 0, ordonnees, 0,
					nombrePointsTraces);
		}
	}

	/**
	 * Fonction appele quand on change le nombre de points du lieu qui est le
	 * bord de la surface
	 */
	@Override
	public void metAJourTableaux() {
		//+ 4 car si on change les deux lieux en lieu ferm les lieux auront un point de plus
		abscisses = new int[((CLieuDeBase) bord).infoLieu.nombreDePoints
				+ deuxiemeLieu.infoLieu.nombreDePoints + 4];
		ordonnees = new int[((CLieuDeBase) bord).infoLieu.nombreDePoints
				+ deuxiemeLieu.infoLieu.nombreDePoints + 4];
	}

	@Override
	public boolean aPourBord(CLieuDeBase lieu) {
		return ((lieu == bord) || (lieu == deuxiemeLieu));
	}

	@Override
	public boolean confonduAvec(CElementBase p) {
		boolean res = super.confonduAvec(p);
		if (!res)
			return false;
		CSurfaceDeuxLieux pt = (CSurfaceDeuxLieux) p;
		return deuxiemeLieu == pt.deuxiemeLieu;
	}

  @Override
	public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
		return memoriseDependance(super.dependDe(p) || deuxiemeLieu.dependDe(p));
	}

  @Override
	public boolean dependDePourBoucle(CElementBase p) {
		return super.dependDePourBoucle(p) || deuxiemeLieu.dependDePourBoucle(p);
	}

  @Override
	public boolean dependDePourCapture(CElementBase p) {
		return super.dependDePourCapture(p) || deuxiemeLieu.dependDePourCapture(p);
	}

  @Override
	public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		CLieuDeBase lieu = (CLieuDeBase) bord;
		existe = (lieu.existe) && (deuxiemeLieu.existe);
		if (!existe)
			return;
		// Les surfaces dlimitant le bord ne doivent tre formes
		// que d'une seule ligne
		if (!(lieu.contientUneSeuleLigne() && deuxiemeLieu.contientUneSeuleLigne())) {
			existe = false;
			return;
		}

		int nombrePointsTracesPremierLieu = lieu.infoLignes[lieu.indicePremiereLigne].nombrePoints;
		int nombrePointsTracesDeuxiemeLieu = deuxiemeLieu.infoLignes[deuxiemeLieu.indicePremiereLigne].nombrePoints;
		nombrePointsTraces = nombrePointsTracesPremierLieu
				+ nombrePointsTracesDeuxiemeLieu;
		int indicePremierPointLieu1 = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint;
		int indiceDernierPointLieu1 = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint
				+ lieu.infoLignes[lieu.indicePremiereLigne].nombrePoints - 1;
		int indicePremierPointLieu2 = deuxiemeLieu.infoLignes[deuxiemeLieu.indicePremiereLigne].indicePremierPoint;
		int indiceDernierPointLieu2 = deuxiemeLieu.infoLignes[deuxiemeLieu.indicePremiereLigne].indicePremierPoint
				+ deuxiemeLieu.infoLignes[deuxiemeLieu.indicePremiereLigne].nombrePoints
				- 1;
		double x1 = lieu.abscissesCoordonneesPoints[indicePremierPointLieu1];
		double y1 = lieu.ordonneesCoordonneesPoints[indicePremierPointLieu1];
		double x4 = lieu.abscissesCoordonneesPoints[indiceDernierPointLieu1];
		double y4 = lieu.ordonneesCoordonneesPoints[indiceDernierPointLieu1];
		double x3 = deuxiemeLieu.abscissesCoordonneesPoints[indiceDernierPointLieu2];
		double y3 = deuxiemeLieu.ordonneesCoordonneesPoints[indiceDernierPointLieu2];
		double x2 = deuxiemeLieu.abscissesCoordonneesPoints[indicePremierPointLieu2];
		double y2 = deuxiemeLieu.ordonneesCoordonneesPoints[indicePremierPointLieu2];
		pointDebutLieu1.placeEn(x1, y1);
		pointFinLieu2.placeEn(x4, y4);
		// On recopie les coordonnes du premier lieu
		int ideb = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint;
		for (int i = 0; i < nombrePointsTracesPremierLieu; i++) {
			abscisses[i] = lieu.abscissesCoordonneesPoints[ideb + i];
			ordonnees[i] = lieu.ordonneesCoordonneesPoints[ideb + i];
		}
		int deb = deuxiemeLieu.infoLignes[deuxiemeLieu.indicePremiereLigne].indicePremierPoint;
		for (int i = 0; i <= nombrePointsTracesDeuxiemeLieu - 1; i++) {
			// Gros bug corrig lors de la version 1.9.5
			// abscisses[nombrePointsTracesPremierLieu + i] =
			// deuxiemeLieu.abscissesCoordonneesPoints
			//[deuxiemeLieu.infoLignes[deuxiemeLieu.indicePremiereLigne].nombrePoints-
			// i-1];
			// ordonnees[nombrePointsTracesPremierLieu + i] =
			// deuxiemeLieu.ordonneesCoordonneesPoints
			//[deuxiemeLieu.infoLignes[deuxiemeLieu.indicePremiereLigne].nombrePoints-
			// i-1];
			abscisses[nombrePointsTracesPremierLieu + i] = deuxiemeLieu.abscissesCoordonneesPoints[deb
					+ nombrePointsTracesDeuxiemeLieu - i - 1];
			ordonnees[nombrePointsTracesPremierLieu + i] = deuxiemeLieu.ordonneesCoordonneesPoints[deb
					+ nombrePointsTracesDeuxiemeLieu - i - 1];
		}
		pointDebutLieu2.placeEn(x2, y2);
		pointFinLieu1.placeEn(x3, y3);
		// }
		// On ferme le polygone
		// Version 1.9.5 : inutile de refermer le polygone
		// abscisses[nombrePointsTraces] = abscisses[0];
		// ordonnees[nombrePointsTraces] = ordonnees[0];
		segmentDebut.positionne(infoRandom, dimfen);
		segmentFin.positionne(infoRandom, dimfen);
		existe = (segmentDebut.existe) && (segmentFin.existe);
	}

	@Override
	public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    pDC.fillPolygon(abscisses, ordonnees, nombrePointsTraces);
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nombrePointsTraces);
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nombrePointsTraces);
    	}
      else {
	    	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire.basseDefinition, couleurFond);
	    	pDC.setPaint(tp);
	   		pDC.fillPolygon(abscisses, ordonnees, nombrePointsTraces);
      }
    }

	}

	@Override
	public double distancePoint(double xp, double yp, boolean masquage) {
		double dist;
		CLieuDeBase lieu;

		if (!existe || (masquage && masque))
			return -1;
		lieu = (CLieuDeBase) bord;
		dist = lieu.distancePointPourSurface(xp, yp, masquage);
		if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
		dist = deuxiemeLieu.distancePointPourSurface(xp, yp, masquage);
		if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
		dist = segmentDebut.distancePoint(xp, yp, masquage);
		if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
		dist = segmentFin.distancePoint(xp, yp, masquage);
		if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
		return -1;
	}

  @Override
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list)
			throws IOException {
		super.chargeDepuisFlux(inps, list);
		int ind1 = inps.readCInt();
		deuxiemeLieu = (CLieuDeBase) (list.get(ind1, CLieuDeBase.class));
		// On cre les tableaux devant recevoir les coordonnes. Le + 3 est du aux
		// deux points projets
		// sur la droite
		// Le +3 est pour tenir compte des lieux ferms auquels on rajoute un point
		// Modifi pour la version 1.9.5. Seules deux coordonnes sont rajoutes car
		// il est
		// inutile de refermer le polygone
		// Modification pour la version 2.5.
		// Certaines choses pas  faire pour le chargement d'un prototype
		if (list.getClass() != CPrototype.class) {
			abscisses = new int[((CLieuDeBase) bord).infoLieu.nombreDePoints
					+ deuxiemeLieu.infoLieu.nombreDePoints + 2];
			ordonnees = new int[((CLieuDeBase) bord).infoLieu.nombreDePoints
					+ deuxiemeLieu.infoLieu.nombreDePoints + 2];
		}
	}

  @Override
	public void read(MathGraphDataInputStream inps, CListeObjets list)
			throws IOException {
		pointDebutLieu1 = new CPointBase();
		pointDebutLieu2 = new CPointBase();
		pointFinLieu1 = new CPointBase();
		pointFinLieu2 = new CPointBase();
		segmentDebut = new CSegment(list, pointDebutLieu1, pointDebutLieu2);
		segmentFin = new CSegment(list, pointFinLieu1, pointFinLieu2);
		super.read(inps, list);
		int ind1 = inps.readInt();
		deuxiemeLieu = (CLieuDeBase) (list.get(ind1, CLieuDeBase.class));
		// Modification pour la version 2.5.
		// Certaines choses pas  faire pour le chargement d'un prototype
		if (list.getClass() != CPrototype.class) {
			abscisses = new int[((CLieuDeBase) bord).infoLieu.nombreDePoints
					+ deuxiemeLieu.infoLieu.nombreDePoints + 2];
			ordonnees = new int[((CLieuDeBase) bord).infoLieu.nombreDePoints
					+ deuxiemeLieu.infoLieu.nombreDePoints + 2];
		}
	}

  @Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list)
			throws IOException {
		super.write(oups, list);
		int ind1 = list.indexOf(deuxiemeLieu);
		oups.writeInt(ind1);
	}
}
