/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.Complexe;
import types.DimensionFenetre;
import types.ErreurCalculException;
import types.Operateur;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 *
 * @author Yves
 */
// Modifi version 3.0
public class CValeurAssocieeComp extends CObjetBase{
  private Complexe valeurComplexe;  // La valeur
  // private CValeurDynamique valeurDynamiqueAssociee; // NULL ou la valeur dynamique associe
  // private String chaineCalculValeur;
	public CCalculBase calcul;
	protected boolean estConstant;
	private boolean existe;
	// Rajout version 3.1.0 pour optimisation. A la cration ou au charement depuis un flux, sera mis  false
	// puis dans positionne, si dejaPositionne est false, on regardera si la calcul est constant.
	// S'il l'est il ne sera plus recalcul saut si on cahnge la formule
	protected boolean dejaPositionne;
  
  /** Creates a new instance of CValeurAssocieeComp */
  public CValeurAssocieeComp() {
  	valeurComplexe = new Complexe();
  	dejaPositionne = false;
  }
  
  // Ajout version 4.8
  public void initialisePourDependance() {
  	calcul.initialisePourDependance();
  }
  
  /**
   * Constructeur pour cration d'objet
   * @param liste
   */
  public CValeurAssocieeComp(CListeObjets liste) {
  	super(liste);
  	valeurComplexe = new Complexe();
    calcul = null;
    dejaPositionne = false;
    // chaineCalculValeur = "";  	
  }
  
  /** Constructeur  partir d'un autre objet du mme type */
  public CValeurAssocieeComp(CValeurAssocieeComp valInit) {
  	super(valInit.listeProprietaire);
    // valeurComplexe = valInit.valeurComplexe;
  	valeurComplexe = new Complexe(valInit.valeurComplexe);
  	valeurComplexe.x = valInit.valeurComplexe.x;
  	valeurComplexe.y = valInit.valeurComplexe.y;
    calcul = valInit.calcul.getClone(listeProprietaire, listeProprietaire);
    estConstant = valInit.estConstant;
    dejaPositionne = valInit.dejaPositionne;
    existe = valInit.existe;
  }
  
  
  /** Constructeur pour clonage */
  public CValeurAssocieeComp(CListeObjets liste, CCalculBase calculInit, Complexe valeurComplexeInit) {
    super(liste);
    valeurComplexe = new Complexe(valeurComplexeInit);
    calcul = calculInit;
    dejaPositionne = false;
  }
  
  public CValeurAssocieeComp getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    // Modifi version 4.4
    // return new CValeurAssocieeComp(listeCible, calculClone, valeurComplexe);
    CValeurAssocieeComp va = new CValeurAssocieeComp(listeCible, calculClone, valeurComplexe);
    va.dejaPositionne = dejaPositionne;
    va.estConstant = estConstant;
    va.valeurComplexe.x = valeurComplexe.x;
    va.valeurComplexe.y = valeurComplexe.y;
    va.existe = existe;
    return va;
  }
  
  /**
   * Ajout pour la version 3.0
   * Donne un calcul  l'objet et vrifie s'il est constant ou non.
   */
  public void donneCalcul(CCalculBase calc) {
  	calcul = calc;
  	dejaPositionne = false;
  }
  
  /** Fonction associant  l'objet la valeur dynamique vald */
  //public void associeA(CValeurDynamique vald) {
  //  valeurDynamiqueAssociee = vald;
  //}
  public void donneValeur(Complexe c) {
  	valeurComplexe.x = c.x;
  	valeurComplexe.y = c.y;
  	calcul = null;
  	dejaPositionne = true;
  	estConstant = true;
  }
  
  //public void donneChaineCalcul(String s) {
  //  chaineCalculValeur = s;
  //}
  
  /**
   * Renvoie une chaine tant gale au nom de la valeur associe s'il y en a un
   * et sinon la chaine reprsentant le calcul associ
   * @return
   */
  public String chaineInfo()
  {
    return calcul.chaineCalculSansPar(null);
  }  
  
  public boolean existe() {
  	return existe;
  }
  
  // Ajout pour version 3.0
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	if (!dejaPositionne) {
      if (calcul.estConstant()) {
      	estConstant = true;
        existe = calcul.existe(infoRandom);
        if (!existe) return;
        try {
          listeProprietaire.initialiseNombreIterations();
          // Modifi version 3.5.1
          // calcul.resultatComplexe(false, valeurComplexe);
          calcul.resultatComplexe(infoRandom, valeurComplexe);
        }
        catch (ErreurCalculException e) {
          existe = false;
          valeurComplexe.x = 0;
          valeurComplexe.y = 0;
        }
        if (Double.isNaN(valeurComplexe.x) || Double.isInfinite(valeurComplexe.x)
          || Double.isNaN(valeurComplexe.y) || Double.isInfinite(valeurComplexe.y)) {
           existe = false;
        }   
      }
      else estConstant = false;
      dejaPositionne = true;
  	}
    if (!estConstant) {
      existe = calcul.existe(infoRandom);
      if (!existe) return;
      // A revoir en java pour la gestion des erreurs de calcul
      // AnnuleErreurDeCalcul();	// Pour dtecter une ventuelle erreur de
      //                         // calcul lors de l'appel
      try {
        listeProprietaire.initialiseNombreIterations();
        calcul.resultatComplexe(infoRandom, valeurComplexe);
      }
      catch (ErreurCalculException e) {
        existe = false;
        valeurComplexe.x = 0;
        valeurComplexe.y = 0;
      }
      if (Double.isNaN(valeurComplexe.x) || Double.isInfinite(valeurComplexe.x)
        || Double.isNaN(valeurComplexe.y) || Double.isInfinite(valeurComplexe.y)) {
         existe = false;
      }
    }
  }
  
  public boolean dependDe(CElementBase p) {
    return calcul.dependDe(p);
  }
  
  public boolean dependDePourReclassement(CElementBase p) {
    return calcul.dependDePourReclassement(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
   return calcul.dependDePourBoucle(p);
  }
  
  public void rendValeurComplexe(boolean infoRandom, Complexe zRes) {
    zRes.set(valeurComplexe.x, valeurComplexe.y);///Cas d'une constante
  }
  
  public boolean confonduAvec(CObjetBase p) {
    CValeurAssocieeComp pt;
    if (p.getClass() == this.getClass()) {
      pt = (CValeurAssocieeComp) p;
      if (estConstant) return (valeurComplexe.x == pt.valeurComplexe.x) && (valeurComplexe.y == pt.valeurComplexe.y) ;
      else return false;    
    } else return false;
  }
  
  
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return calcul.estDefiniParObjetDans(listeOb);
	}  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     // Plus utulis
     @SuppressWarnings("unused")
		 String chaineCalculValeur = inps.readCString();
     valeurComplexe.x = inps.readCDouble();
     valeurComplexe.y = inps.readCDouble();
     if ( ind1 == -1) {
    	 CConstantei constantei = new CConstantei();
    	 CConstante a = new CConstante(list,valeurComplexe.x);
    	 CConstante b = new CConstante(list,valeurComplexe.y);
    	 COperation op1 = new COperation(list,  b, constantei, Operateur.Mult);
    	 calcul = new COperation(list,  a, op1, Operateur.Plus);
    	 estConstant = true;
    	 existe = true;
     }
     else {
    	 CValeurDynamique valeurDynamiqueAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
    	 calcul = new CResultatValeurComplexe(list, valeurDynamiqueAssociee);
    	 estConstant = false;    	 
     }
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    calcul = (CCalculBase) (inps.readObject(list));
  	// Modifi version 3.1.5
  	// Il ne faut pas faire appel  existe car par exemple si le calcul est f(3) o f
  	// est une fonction utilisateur, f n'a pas encore t positionne
    /*
    if (calcul.estConstant()) {
    	estConstant = true;
      existe = calcul.existe();
      if (!existe) return;
      try {
        listeProprietaire.initialiseNombreIterations();
        calcul.resultatComplexe(false, valeurComplexe);
      }
      catch (ErreurCalculException e) {
        existe = false;
        valeurComplexe.x = 0;
        valeurComplexe.y = 0;
      }
      if (Double.isNaN(valeurComplexe.x) || Double.isInfinite(valeurComplexe.x)
        || Double.isNaN(valeurComplexe.y) || Double.isInfinite(valeurComplexe.y)) {
         existe = false;
      }   
    }
    */
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeObject(calcul);
  }
  
  /**
   * Ajout pour la version 3.0 : renvoie true seulement si la valeur est constante
   * ou si la valeur dynamique associe est constante
   * @return
   */
	public boolean estConstant() {
		return estConstant;
	}
  
}
