/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CVecteur extends CSegment {
  public int motifFleche;
  /** Creates new CVecteur */
  public CVecteur() {
  }
  
  // Constructeur pour clonage
  public CVecteur(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleInit,
      CPointAncetre point1Init, CPointAncetre point2Init, int motifFlecheInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, point1Init, point2Init);
    motifFleche = motifFlecheInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    int ind2 = listeSource.indexOf(point1);
    int ind3 = listeSource.indexOf(point2);
    CElementBase ptelb = new CVecteur(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre) listeCible.get(ind2, CPointAncetre.class),
      (CPointAncetre) listeCible.get(ind3, CPointAncetre.class), motifFleche);
    return ptelb;
  }
  
  @Override
  public String complementInfo() {
    return getString("chinfo155");
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo154");
  }
  
  @Override
  /**
   * A t modifi par rapport au C++
   */
	public boolean confonduAvec(CElementBase p) {
    CVecteur pt;
    if (p.getClass() == this.getClass()) {
      pt = (CVecteur) p;
  		return 
			(point1 == pt.point1) && (point2 == pt.point2);
   }
    else return false;
	}
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	if (horsFenetre) return;
    boolean basseDefinition = listeProprietaire.basseDefinition;
    // Modification version 3.2.0
    // if (!(point1.horsEcran) && !(point2.horsEcran) && vecteur.nonNul())
    if (vecteur.nonNul())
    {
      super.trace( pDC, couleurFond, transparence);
      int styleTrait;
      if (masque) styleTrait = StyleTrait.traitPointille;
      else styleTrait = style;
      pDC.setStroke(Pinceau.CreePinceau(styleTrait));
      pDC.setColor(couleur);
      // On ne trace la flche que si le vecteur n'est pas presque nul
      if (!vecteur.presqueNul()) StyleFleche.traceFleche(pDC,x1,y1,x2,y2,motifFleche, basseDefinition);
    }
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NVecteur;
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationVecteur");
  }
  
  public void donneStyleFleche(int styleFleche) {
  	motifFleche = styleFleche;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    motifFleche = inps.readByte();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    motifFleche = inps.readByte();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motifFleche);
  }  
}
