/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author Yves
 * Classe ancetre de tous les objets outils de cration d'objet
 */
package outils;

import javax.swing.*;

import entreesSorties.MathGraphTimerClig;

import objets.*;

import types.*;

import java.awt.*;
import Applet.*;
import java.util.ListIterator;

public abstract class Outil {
  AppletMathGraph32 appletMere;
  ImageIcon iconeAssociee = null ;
  public boolean nbClignPair;
  private boolean avecClignotement;
  MathGraphTimerClig timerTaskClignotement;  
  java.util.Timer timer;
	// Ajout version 4.8 pour les outils de construction itrative et rcursive
	public final static int sansLigne = 0;
	public final static int avecPolygone = 1;
	public final static int avecLigneBrisee = 2;

	/** Creates a new instance of Outil */
  public Outil(AppletMathGraph32 appletMereInit, boolean avecClignotementInit, ImageIcon iconeAssocieeInit) {
    appletMere = appletMereInit;
    avecClignotement = avecClignotementInit;
    iconeAssociee = iconeAssocieeInit;
  }
  
  public static final String getString(String key) {
  	return AppletMathGraph32.getString(key);
  }
  
  /**
   * Dfinition des constantes C++ associes aux items de menus
   */
  public int idMenu() {
    return 0;
  }
  
  /**
   * Fonction renvoyant une chane destine  tre affiche dans la ligne
   * d'information pour donner la fonction de l'outil
   * @return
   */
  public String chaineInfo() {
    return "";
  }

          
  public String toolTip() {
    return "";
  }
  /**
   * 
   * @return Renvoie un pointeur sur le bouton correspondant  l'outil cr
   * ou null si l'outil est interdit pour la figure
   */
  public MtgButton boutonAssocie() {
    // Si l'item de l'outil est 0, c'est que le bouton correspondant
    // doit figurer toujours dans la barre d'outils
    int id = idMenu();
    CMathGrafDoc doc = appletMere.document;
    boolean b = doc.itemContenu(id);
    if ((id == 0) 
           || (!doc.itemsCochesInterdits && b)
           || (doc.itemsCochesInterdits && !b)) {
    	/* Modifi vesion 4.7.4
      URL adr = appletMere.getClass().getResource("images/" + nomGif());
      ImageIcon image = new ImageIcon(adr);
      MtgButton button = new MtgButton(appletMere, image, this);
      */
      MtgButton button = new MtgButton(appletMere, iconeAssociee, this);
      button.setToolTipText(toolTip());
      return button;
    }
    else return null;
  }
 
  /** Appel lorsque l'outil est slectionn */
  public void select() {
  	// Ligne indispensable pour que le bouton slectionn ne garde pas un cadre pointill quand il est dselectionn
  	appletMere.requestFocus();
    appletMere.outilActif.deselect();
    if (avecClignotement) {
      timerTaskClignotement = new MathGraphTimerClig(appletMere);
      timer = new java.util.Timer();
      timer.schedule(timerTaskClignotement, 0l, 500); 
      resetClignotement();
    }
    appletMere.listeExclusionDesignation.retireTout();
    appletMere.listeObjetsVisuels.retireTout();
    appletMere.outilPointageActif = appletMere.outilPointageCre;
    appletMere.outilActif = this;
  }
  
  void designationDe(long nature) {
    appletMere.outilPointageActif.aDesigner = nature;
  }
  
  /** Appel lorsque l'outil est dslectionn */
  /*
  public void deselect() {
    if (avecClignotement) {
      annuleClignotement();
      if (timer != null) timer.cancel();
      if (timerTaskClignotement != null)  timerTaskClignotement.cancel();
      timerTaskClignotement = null;
      timer = null;
      // La ligne suivante est indispensable car dans certains cas des points ont t nomms
      // pour des outils de mesure et quand on annule la cration de mesure il faut que leur nom
      // soit effac. Pour cela on efface toute la fentre
      appletMere.redessineSansCalcul();
    }
    System.gc();
    // appletMere.redessine();
    // appletMere.activeOutilCapture();
  }
  */
  
	/** Appel lorsque l'outil est dslectionn */
	public void deselect() {
		appletMere.videDesignation();
		designationDe(0);
		if (avecClignotement) {
			annuleClignotement();
			if (timer != null)
				timer.cancel();
			if (timerTaskClignotement != null)
				timerTaskClignotement.cancel();
			timerTaskClignotement = null;
			timer = null;
			appletMere.repaintFigureSansCalcul();
		}
		else annuleObjetsVisuels();
		System.gc();
	}  
  
  /** Mthode renvoyant true seulement si l'outil peut tre slectionn.
   * A redfinir seulement pour les outils qui peuvent ventuellement tre
   * impossibles  slectionner, par exemple l'outil de cration d'un cercle
   * par son centre et son rayon si une longueur unit n'a pas t dfinie pour
   * la figure
  **/
  public boolean activationValide() {
    return true;
  }
  /** Appel pour initialiser le clignotement */
  public void resetClignotement() {
    nbClignPair = true;
		appletMere.outilPointageActif.aRedessinerDansPaint = true;
  }

  /** Appel pour arrter le clignotement */
  public void annuleClignotement() {
		appletMere.outilPointageActif.aRedessinerDansPaint = true;
    MtgPaneFigure pane = appletMere.paneFigure;
    appletMere.listeClignotante.retireTout();
    appletMere.listeObjetsVisuels.retireTout();
    pane.repaint(); //  revoir
  }
  
	/** Appel pour mettre  zro la liste visuelle et rafficher la figure
	 */
	public void annuleObjetsVisuels() {
		appletMere.listeObjetsVisuels.retireTout();
		appletMere.repaintFigureSansCalcul();
	}
  
  /* Appel pour ajouter un lment graphqie  la liste clignotante de l'applet */
  public void ajouteClignotementDe(CElementGraphique el) {
    appletMere.listeClignotante.add(el);
  }
  /** Appel pour exclure une lment des objets qui peuvent tre dsigns  la souris */
 public void excluDeDesignation(CElementGraphique el) {
   appletMere.listeExclusionDesignation.add(el);
 }
  /* Appel lorsqu'un objet a t cliqu pour tre trait par l'outil
   * par exemple le premier point pour crer une droite passant par deux points */
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  }
  /* Appel pour crer finalement l'objet une fois que tous les objets dont il
   * dpend ont t dsigns */
  public void creeObjet() {
  }
  
	/**
	 * A redfinir pour les objets qui crent l'objet aprs un clic droit
	 */
	public void creeObjetApresClicDroit() {
	}
  
  /* Verification de l'existence de l'objet point par pto
   * Si l'objet existe dj, un message d'avertissement est affich
   * et sinon l'objet pto est rajout  la liste principale de l'applet */
  public void ajouteElement(CElementBase pto) {
    CListeObjets liste = appletMere.listePr;
    if (!liste.existeDeja(pto)) {
      appletMere.ajouteElement(pto);
      appletMere.positionne();
      appletMere.repaint();
    }
  }
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  public void ajouteObjetsVisuels() {
  }
  
  /** Mthode ajoutant un rapporteur  la liste d'objets visuels.
   *  Utilise pour l'outil rapporteur et peut-tre plus tard les
   *  outils de cration d'arc de cercle.
   *Renvoie un pointeur sur un point d'intersection entre le cercle extrieur
   * du rapporteur et une demi droite joignant le centre de ce cercle avec
   * le point suivant le pointeur souris
   */
	public CPointAncetre ajouteRapporteur(CPointAncetre origine) {
		// On cre d'abord un cercle de centre origine et de rayon fixe
		// qui servira de support extrieur au rapporteur
		int styleTrait;
		CListeObjets liste = appletMere.listeObjetsVisuels;
		CCercle ptCercleRapExt = new CCercle(liste, null, false, Couleur
				.TraduitCouleur(1), false, StyleTrait.traitFin, origine.x, origine.y,
				80);
		appletMere.listeObjetsVisuels.add(ptCercleRapExt);
		// Puis un cercle intrieur
		CCercle ptCercleRapInt = new CCercle(liste, null, false, Couleur
				.TraduitCouleur(1), false, StyleTrait.traitFin, origine.x, origine.y,
				40);
		appletMere.listeObjetsVisuels.add(ptCercleRapInt);
		// On cre un demi-droite de Origine vers PointSouris
		CDemiDroiteOA ptdemid = new CDemiDroiteOA(liste, null, false, Couleur
				.TraduitCouleur(1), false, StyleTrait.traitPointille, origine, appletMere.pointSouris);
		appletMere.listeObjetsVisuels.add(ptdemid);
		// On cre l'intersection de cette demi-droite avec le cercle extrieur
		// du rapporteur
		CIntDroiteCercle ptint = new CIntDroiteCercle(liste, null, false, ptdemid,
				ptCercleRapExt);
		appletMere.listeObjetsVisuels.add(ptint);
		CPointAncetre ptplb = new CPointLieBipoint(liste, null, false, Couleur
				.TraduitCouleur(1), false, 0d, 0d, false, "", 2, MotifPoint.petitRond,
				ptint, 1, false);
		ptplb.cache();
		appletMere.listeObjetsVisuels.add(ptplb);
		// On rajoute des graduations au rapporteur
		// On cre d'abord une rotation d'angle 10
		CRotation ptrot10 = new CRotation(liste, null, false, origine, 10);
		appletMere.listeObjetsVisuels.add(ptrot10);
		// On cre une homothtie servant  crer les graduations de 10
		CHomothetie hom1 = new CHomothetie(liste, null, false, origine, 0.85);
		appletMere.listeObjetsVisuels.add(hom1);
		// Et une autre pour les graduations de 90
		CHomothetie hom2 = new CHomothetie(liste, null, false, origine, 0.5);
		appletMere.listeObjetsVisuels.add(hom2);
		// On cre les images par des rotations d'angle 10
		CPointAncetre ptp1 = ptplb;
		for (int i = 1; i <= 36; i++) {
			CPointImage ptim = new CPointImage(liste, null, false, Couleur
					.TraduitCouleur(3), MotifPoint.pixel, 2, ptp1, ptrot10);
			ptim.cache();
			appletMere.listeObjetsVisuels.add(ptim);
			// On cre l'image du point cr par l'homothtie pour crer les
			// graduations
			// Pour les multiples de 90 elles sont rouges, sinon vertes
			CPointImage ptima = new CPointImage(liste, null, false, Couleur
					.TraduitCouleur(3), 2, MotifPoint.pixel, ptim, hom1);
			ptima.cache();
			appletMere.listeObjetsVisuels.add(ptima);
			// On cre un segment graduation
			int coul;
			if ((i % 9) == 0) {
				coul = 4;
				styleTrait = StyleTrait.traitEpais;
			} else {
				coul = 2;
				styleTrait = StyleTrait.traitFin;
			}
			// On cre des graduations joignant le centre aux 90
			if ((i % 9) == 0) {
				CPointImage ptim2 = new CPointImage(liste, null, false, Couleur
						.TraduitCouleur(1), MotifPoint.petitRond, 2, ptim, hom2);
				ptim2.cache();
				appletMere.listeObjetsVisuels.add(ptim2);
				CSegment ptseg90 = new CSegment(liste, null, false, origine, ptim2); // Segment
																																							// en
																																							// pointills
				appletMere.listeObjetsVisuels.add(ptseg90);
				CPointImage ptim3 = new CPointImage(liste, null, false, Couleur
						.TraduitCouleur(1), MotifPoint.petitRond, 2, ptima, hom1);
				ptim3.cache();
				appletMere.listeObjetsVisuels.add(ptim3);
				String chai = new String();
				switch (i) {
				case 9:
				case 27:
					chai = "90";
					break;
				case 18:
					chai = "180";
					break;
				case 36:
					chai = "0";
					break;
				}
				CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
						.TraduitCouleur(coul), 0d, 0d, 0d, 0d, false, ptim3, 0,
						StyleEncadrement.Sans, false, Couleur.TraduitCouleur(0), chai,
            CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);

				// Version C++ : ptcom->DecaleNom(-75,-20);
				appletMere.listeObjetsVisuels.add(ptcom);
			}
			CSegment ptgrad = new CSegment(liste, null, false, Couleur
					.TraduitCouleur(coul), false, styleTrait, ptima, ptim);
			appletMere.listeObjetsVisuels.add(ptgrad);
			// Pour 10 et -10 on ajoute un commentaire de graduation
			if ((i == 1) || (i == 35)) {
				CPointImage ptim4 = new CPointImage(liste, null, false, Couleur
						.TraduitCouleur(1), MotifPoint.petitRond, 2, ptima, hom1);
				ptim4.cache();
				appletMere.listeObjetsVisuels.add(ptim4);
				CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
						.TraduitCouleur(coul), 0d, 0d, -10, -6, false, ptim4, 0,
						StyleEncadrement.Sans, false, Couleur.TraduitCouleur(0), "10",
		        CValeurAffichee.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
				appletMere.listeObjetsVisuels.add(ptcom);
			}
			ptp1 = ptim;
		}
		return ptplb;
	}

  
	/**
	 * Fonction crant l'image de objet par la transformation ptTRans suivant la
	 * nature de cet objet. L'objet appartient  la liste d'objets liste
	 */
	CElementGraphique imageParTransformation(CListeObjets liste,
			CElementGraphique objet, CTransformation trans, Int nombreObjetsAjoutes) {
		nombreObjetsAjoutes.setValue(0);
		CElementGraphique elg = null;
		long nat = objet.getNature();
		Color coul = appletMere.couleurActive();
		if ((nat & NatureObjet.NToutPoint) != 0) {			
			elg = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, "",
					2, MotifPoint.petitRond, (CPointAncetre) objet, trans, false);
		} else {
			if (nat == NatureObjet.NDroite) {
				elg = new CDroiteImage(liste, null, false, coul, false, 0d, 0d, false,
						"", 2, StyleTrait.traitFin,
						CDroite.abscisseNomParDefaut, (CDroite) objet, trans);
			} else {
				if (nat == NatureObjet.NDemiDroite) {
					elg = new CDemiDroiteImage(liste, null, false, coul, false, StyleTrait.traitFin, (CDemiDroite) objet, trans);
				} else {
					if (nat == NatureObjet.NCercle) {
						elg = new CCercleImage(liste, null, false, coul, false, StyleTrait.traitFin, (CCercle) objet, trans);
					} else {
						if (nat == NatureObjet.NArc) {
							elg = new CArcDeCercleImage(liste, null, false, coul, false,
									StyleTrait.traitFin, (CArcDeCercleAncetre) objet, trans);
						} else {
							if (nat == NatureObjet.NSegment) {
								CSegment seg = (CSegment) objet;
								CPointImage ptim1, ptim2;
								ptim1 = liste.pointImageDePar(seg.point1, trans);
								if (ptim1 == null) {
									ptim1 = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, "",
											2, MotifPoint.petitRond, seg.point1, trans, false);
									appletMere.ajouteElement(ptim1);
									nombreObjetsAjoutes.setValue(nombreObjetsAjoutes.getValue() + 1);
								}
								ptim2 = liste.pointImageDePar(seg.point2, trans);
								if (ptim2 == null) {
									ptim2 = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, "",
											2, MotifPoint.petitRond, seg.point2, trans, false);
									appletMere.ajouteElement(ptim2);
									nombreObjetsAjoutes.setValue(nombreObjetsAjoutes.getValue() + 1);
								}
								elg = new CSegment(appletMere.listePr, null, false,
										appletMere.couleurActive(), false, StyleTrait.traitFin,
										ptim1, ptim2);

							} else {
								if ((nat & (NatureObjet.NLigneBrisee | NatureObjet.NPolygone)) != 0) {
									CLignePolygonaleAncetre ligneanc = (CLignePolygonaleAncetre) objet;
									CPointImage ptim;
									if (nat == NatureObjet.NLigneBrisee)
										elg = new CLigneBrisee(liste, appletMere.couleurActive(), StyleTrait.traitFin);
									else
										elg = new CPolygone(liste, appletMere.couleurActive(), StyleTrait.traitFin);
									CLignePolygonaleAncetre ligne = (CLignePolygonaleAncetre) elg;
									ListIterator<CNoeudPointeurSurPoint> li = ligneanc.collectionPoints.listIterator();
									while (li.hasNext()) {
										CPointAncetre ant = (CPointAncetre) (li.next()).pointeurSurPoint;
										ptim = liste.pointImageDePar(ant, trans);
										if (ptim == null) {
											ptim = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, "",
													2, MotifPoint.petitRond, ant, trans, false);
											appletMere.ajouteElement(ptim);
											nombreObjetsAjoutes.setValue(nombreObjetsAjoutes.getValue() + 1);
										}
										ligne.collectionPoints.add(new CNoeudPointeurSurPoint(liste, ptim));
									}
									// Il faut prparer les tableaux de coordonnes une fois que l'on connait le nombre de points de la ligne
									ligne.prepareTableauxCoordonnees();
								}
							}
						}
					}
				}
			}
		}
		return elg;
	}
  /**
   * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme indication
   * Doit tre redfinie par les descendants devant crire plusieurs indications
   * @param i : l'indice
   * @return
   */
          
  public String chaineIndication (int indEtape) {
    return "";
  }
  
	/**
	 * Renvoie le curseur utilis par l'outil quand le pointeur soursi entre dans
	 * une fentre MtgInternalFrame Est par dfaut une petitRond. A redfir pour les
	 * outils utilisant un autre curseur
	 * 
	 * @return
	 */
	public Cursor curseurUtilise() {
		return appletMere.outilPointageActif.curseurActif;
	}
  
}
