/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import Applet.AppletMathGraph32;
import java.awt.Point;

import dialogs.RapportHomothetieDlg;

import objets.*;

import types.Couleur;
import types.Int;
import types.MotifPoint;
import types.NatureObjet;

/**
 * Outil de crtion d'images par une homothtie
 * @author Yves
 */
public class OutilHom extends Outil {
  CPointAncetre point;
  CElementGraphique objet;
  private int nombreImagesCrees;
  CHomothetie ptHom;
  
  /** Creates a new instance of OutilRot */
  public OutilHom(AppletMathGraph32 appletMereInit) {
    super(appletMereInit, true, AppletMathGraph32.iconCreerImageParHomothetie);
  }

  public int idMenu() {
    return 32801;
  }  

  public String toolTip() {
    return getString("toolTip26");
  }
  
  /** Appel lorsque l'outil est slectionn */
  public void select() {
    super.select();
    point = null;
    objet = null;
    nombreImagesCrees = 0;
    designationDe(NatureObjet.NToutPoint);
    appletMere.indication(getString("homothetie1"));
  }
  
  /**
   * Il est ncessaire de regarder quand l'outil est dselectionn si on a vraiment cr des images
   * par une transformation. Sinon il faut supprimer le dernier objet qui est une homothtie
   */
  @Override
  public void deselect() {
    CListeObjets liste = appletMere.listePr;
    CElementBase elb = liste.get(liste.size() - 1);
    if (elb.getNature() == NatureObjet.NTransformation) {
    	appletMere.detruitDernierElement();
    }
    super.deselect();
  }
  

  @Override
  public void creeObjet() {
    // CListeObjets liste = appletMere.listePr;
  	CListeObjets liste = appletMere.listeConstruction;  // Modifi version 4.6
    if (nombreImagesCrees == 0) {
    	CTransformation hom = liste.transformationConfondueAvec(ptHom);
    	if (hom != null) {
    		ptHom = (CHomothetie) hom;
     	}
    	else appletMere.ajouteElement(ptHom);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, ptHom, nbImagesCrees);
    if (appletMere.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), ptHom)) {
    	appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil22"));
    	nombreImagesCrees++;
  		appletMere.repaintFigureAvecCalcul();    	
    }
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CListeObjets liste = appletMere.listePr;
    if (point == null) {
      appletMere.indication(getString("homothetie2"));
      point = (CPointAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
      CValeurAssocieeAVariable rap = new CValeurAssocieeAVariable(liste, 0);
      ptHom = new CHomothetie(liste, null, false, point, rap);
      // On cre un dialogue modal permettant d'entrer l'angle de l'homothtie
      RapportHomothetieDlg rapportDlg = new RapportHomothetieDlg(appletMere, ptHom);
      rapportDlg.setLocationRelativeTo(appletMere); // Pour centrer la boite de dialogue sur l'cran
      // On redessine la figure pour liminer la dsignation ce point affiche pour le centre
      appletMere.videDesignation();
      appletMere.repaintFigureSansCalcul();
      rapportDlg.setVisible(true);
      // On regarde si la bote de dialogue a bien t valide par OK
      if (rapportDlg.retourOK()) {
        ajouteObjetsVisuels();
        designationDe(appletMere.natureObjetsPourImages());
        resetClignotement();
        appletMere.indication(getString("homothetie3"));
      }
      else {
        select(); 
      }      
    }
    else {
      objet = elg;
      creeObjet();
    }
  }      
  /* Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  public void ajouteObjetsVisuels() {
    CListeObjets liste = appletMere.listeObjetsVisuels;
    appletMere.ajouteObjetVisuel(ptHom);
    CPointImage ptim = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
      MotifPoint.petitRond, 2, appletMere.pointSouris, ptHom);
    appletMere.ajouteObjetVisuel(ptim);
  	// On cre un segment en pointills joignant le centre au point souris
  	CSegment pts1 = new CSegment(liste, null, false, point, ptim);
    appletMere.ajouteObjetVisuel(pts1);
    // Ajout version 4.5.2 : Si le rapport est ngatif on joint l'origine au point souris
    ptHom.positionne(appletMere.infRand, appletMere.dimf);
    if (ptHom.rapport.rendValeur(false) < 0) {
    	CSegment pts2 = new CSegment(liste, null, false, point, appletMere.pointSouris);
      appletMere.ajouteObjetVisuel(pts2);
    }
  }
}
