/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import Applet.AppletMathGraph32;
import java.awt.Color;
import objets.*;
import types.Couleur;
import types.MotifPoint;
import types.StyleEncadrement;
import types.StyleFleche;
import types.StyleTrait;

/**
 *
 * @author Yves
 */
public class OutilMesLong extends OutilPar2pts{

  /** Creates a new instance of OutilSeg */
  public OutilMesLong(AppletMathGraph32 appletMereInit) {
    super(appletMereInit, AppletMathGraph32.iconMesureLongueur);
  }

  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soeint nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  boolean nomsPointsIndisp() {
    return true;
  }
  
  public int idMenu() {
    return 32813;
  }
  
  public String toolTip() {
    return getString("toolTip28");
  }
  
  /* Appel pour crer finalement l'objet une fois que tous les objets dont il
   * dpend ont t dsigns */
  public void creeObjet() {
    appletMere.videDesignation();
    CLongueur lon = new CLongueur(appletMere.listePr, null, false , point1, point2);
    if (appletMere.ajouteElementAvecVerification(lon)) {
      objetCree = true; // Necessaire  cause du fait que point1 et point2 ont t ventuellement
      // nomms et que si on ne cre pas l'objet on remet alors leur nom  une chaine vide
      appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil23"));
      appletMere.dialogueAvertissement(getString("ch25") + " " + lon.nom() + " " + getString("ch26"));
    }
    select();
  }
  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  public void ajouteObjetsVisuels() {
    CListeObjets list = appletMere.listeObjetsVisuels;
    // On cre deux vecteurs provisoires pour avoir une double flche
    CVecteur v1 = new CVecteur(list, null, false,
            appletMere.couleurActive(), false, StyleTrait.traitPointille,
            point1, appletMere.pointSouris, StyleFleche.FlecheLonguePleine);
    appletMere.listeObjetsVisuels.add(v1);
    CVecteur v2 = new CVecteur(list, null, false,
            appletMere.couleurActive(), false, StyleTrait.traitPointille,
            appletMere.pointSouris, point1, StyleFleche.FlecheLonguePleine);
    appletMere.listeObjetsVisuels.add(v2);
    // On cre le milieu pour lui lier un point d'interrogation     
    CMilieu mil = new CMilieu(list, null, false, Color.BLACK,
            MotifPoint.petitRond, appletMere.pointSouris, point1);
    mil.cache();
    appletMere.listeObjetsVisuels.add(mil);
    // On cre un commentaire contenant comme texte un point d'interrogation
    CCommentaire com = new CCommentaire(list, null, false, Color.BLACK, 0d,0d,
      0d, 0d, false, mil, 1, StyleEncadrement.Sans, false, Couleur.TraduitCouleur(0), "?",
      CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
  appletMere.listeObjetsVisuels.add(com);
  }
  
  /**
   * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme indication
   * Doit tre redfinie par les descendants devant crire plusieurs indications
   * @param i : l'indice
   * @return
   */          
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("mesLong1");
      case 2 : return getString("mesLong2");
      default: return "";
    }
  }  
}
