/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package types;
/*
  *
 * Created on 5 juillet 2004, 23:05
 */

/**
 *
 * @author  Yves
 * @version 
 */
// Classe dfinissant les diffrents styles de flches pour les vecteurs et les arcs
// Se charge aussi de tracer les flches au bout des vecteurs et des arcs

import glob.Kernel;

import java.awt.*;
public abstract class StyleFleche {

  public static final int FlecheLonguePleine = 0;
  public static final int FlecheCourtePleine = 1;
	public static final int FlecheLongueSimple = 2;
  public static final int FlecheCourteSimple = 3;
  
  public static final int longeurFlechePourVecteur(int motif, boolean basseResolution) {
  	if (basseResolution) {
	    switch (motif)
	    {
	      case FlecheLonguePleine:
	      case FlecheLongueSimple:
	          return 8;
	      case FlecheCourtePleine:
	      case FlecheCourteSimple:
	          return 6;
	      default : return 4;
	    }
  	}
  	else {
	    switch (motif)
	    {
	      case FlecheLonguePleine:
	      case FlecheLongueSimple:
	          return 80;
	      case FlecheCourtePleine:
	      case FlecheCourteSimple:
	          return 60;
	      default : return 40;
	    }
  	}
  }  
  public static final int longeurFlechePourArc(int motif, boolean basseResolution) {
  	if (basseResolution) {
	    switch (motif)
	    {
	      case FlecheLonguePleine:
	      case FlecheLongueSimple:
	          return 6;
	      case FlecheCourtePleine:
	      case FlecheCourteSimple:
	          return 4;
	      default : return 4;
	    }
  	}
  	else {
	    switch (motif)
	    {
	      case FlecheLonguePleine:
	      case FlecheLongueSimple:
	          return 60;
	      case FlecheCourtePleine:
	      case FlecheCourteSimple:
	          return 40;
	      default : return 40;
	    }
  		
  	}
  }
  public static final void traceFleche(Graphics2D pDC, double xa, double ya, double xb,
      double yb,int motifFleche, boolean basseResolution)
  {
    int[] abscisses = new int[3];
    int[] ordonnees = new int[3];
    VecteurDirecteur u0 = new VecteurDirecteur();
    VecteurDirecteur u1 = new VecteurDirecteur();
    VecteurDirecteur u2 = new VecteurDirecteur();
    double longueurFleche = StyleFleche.longeurFlechePourVecteur(motifFleche, basseResolution);

    u0.setVecteur(xb, yb, xa, ya);
    u0.vecteurColineaire(longueurFleche,u1);
    // On fait tourner le vecteur u1 de -pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 - u1.y*Kernel.sin30;
    u2.y = u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[1] = (int)Math.round(xb);
    ordonnees[1] = (int)Math.round(yb);
    abscisses[0] = (int)Math.round(xb + u2.x);
    ordonnees[0] = (int)Math.round(yb + u2.y);
    // On fait tourner le vecteur u1 de pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 + u1.y*Kernel.sin30;
    u2.y = -u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[2] = (int)Math.round(xb + u2.x);
    ordonnees[2] = (int)Math.round(yb + u2.y);
    switch (motifFleche)
    {
      case FlecheLonguePleine:
      case FlecheCourtePleine:
        // Il faut tracer le polygone car fillPolygon ne trace que l'intrieur'
        pDC.drawPolygon(abscisses, ordonnees, 3);
        pDC.fillPolygon(abscisses, ordonnees, 3);
        break;
      case FlecheLongueSimple:
      case FlecheCourteSimple:
        pDC.drawPolyline(abscisses, ordonnees, 3);
    }
  }
  // Procdure dessinant la flche en bout d'une marque d'angle orient
  // Le vecteur u dsigne la direction oppose  celle de la flche
  // et (xa, ya) le point extrmit de la flche
  public static final void traceFlechePourArc(Graphics2D pDC, double xa, double ya,
      VecteurDirecteur u,int motifFleche, boolean basseResolution) {
    int[] abscisses = new int[3];
    int[] ordonnees = new int[3];
    VecteurDirecteur u1 = new VecteurDirecteur();
    VecteurDirecteur u2 = new VecteurDirecteur();
    double longueurFleche = StyleFleche.longeurFlechePourArc(motifFleche, basseResolution);

    u.vecteurColineaire(longueurFleche,u1);
    // On fait tourner le vecteur u1 de -pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 - u1.y*Kernel.sin30;
    u2.y = u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[1] = (int)Math.round(xa);
    ordonnees[1] = (int)Math.round(ya);
    abscisses[0] = (int)Math.round(xa + u2.x);
    ordonnees[0] = (int)Math.round(ya + u2.y);
    // On fait tourner le vecteur u1 de pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 + u1.y*Kernel.sin30;
    u2.y = -u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[2] = (int)Math.round(xa + u2.x);
    ordonnees[2] = (int)Math.round(ya + u2.y);
    switch (motifFleche)
    {
      case FlecheLonguePleine:
      case FlecheCourtePleine:
        // Il faut tracer le polygone car fillPolygon ne trace que l'intrieur'
        pDC.drawPolygon(abscisses, ordonnees, 3);
        pDC.fillPolygon(abscisses, ordonnees, 3);
        break;
      case FlecheLongueSimple:
      case FlecheCourteSimple:
        pDC.drawPolyline(abscisses, ordonnees, 3);
    }
  }  
}
